#! /usr/bin/env python
#
# SismoVi, version 2.00, December 2005.
#      
# Copyright (C) 2004-2005, Francesco Apostolico, Geza Seriani
# This program is free software you can redistribute it and/or 
#     modify it under the terms of the GNU General Public License 
#     as published by the Free Software Foundation either version 2
#     of the License, or any later version, with the following conditions
#     attached in addition to any and all conditions of the GNU
#     General Public License:
#     When reporting or displaying any results or animations created
#     using this code or modification of this code, make the appropriate
#     citation referencing SismoVi by name and including the version
#     number.
#
# This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY without even the implied warranty 
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#     along with this program if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
#     02111-1307  USA
#
# Authors:
#
# Francesco Apostolico, Geza Seriani
# Istituto Nazionale di Oceanografia e di Geofisica Sperimentale - OGS
# Borgo Grotta Gigante, 42/C, Sgonico
# 34010, Trieste, Italy
#
# fapostolico@inogs.it, gseriani@inogs.it
# 
# #######################################/
# SismoVi is a tool for the pre and post processing of data that are used in
#      or generated by seismic modeling.
#      It is platform independent and it is based on 
#      Python 2.3 (http://www.python.org/),
#      numarray 1.3 (http://www.stsci.edu/resources/software_hardware/numarray) and
#      WxPython 2.6 (http://wxpython.org/)
#      that must be installed in your platform before running it. 
#      An interactive help is available when running SismoVi.
#---------------------------------------------------------------------------
###### SismoVi.py

import os
import sys
import wx
import geomodvi
import meshvi
import datavi
import commonvi as svcom
import wx.tools.helpviewer as helpv

class SismoVi(wx.Frame):
    def OnSplashVi(self, event):
        def Destroy(event):
            splashframe.Destroy()
        splashframe=wx.MiniFrame(self, -1, size=(398,308), style= wx.SIMPLE_BORDER | wx.FRAME_NO_TASKBAR | wx.STAY_ON_TOP)
        splashframe.CentreOnScreen()
        img=wx.StaticBitmap(splashframe,-1,wx.Image(self.path+self.resfolder+"splash.gif", wx.BITMAP_TYPE_GIF).ConvertToBitmap())
        splashframe.Show(True)
        img.Bind(wx.EVT_LEFT_DOWN, Destroy)
    
    def OnGeomdVi(self, event):
        self.counter_geomodvi+=1
        x0,y0=self.GetPositionTuple()
        x1,y1=self.GetSizeTuple()
        geomodvi.GeomodDesignFrame(parent=self, pos=(x0+x1+1,y0), size=(800,600), counter=" - win # "+str(self.counter_geomodvi), path=self.path)
    
    def OnMeshVi(self, event):
        self.counter_meshvi+=1
        x0,y0=self.GetPositionTuple()
        x1,y1=self.GetSizeTuple()
        meshvi.MeshParamsDialog(parent=self, pos=(x0+x1+1,y0), size=(800,600), counter=" - win # "+str(self.counter_meshvi), path=self.path)
    
    def OnDataVi(self, event):
        self.counter_datavi+=1
        x0,y0=self.GetPositionTuple()
        x1,y1=self.GetSizeTuple()
        datavi.DataParamsDialog(parent=self, pos=(x0+x1+1,y0), size=(800,600), counter=" - win # "+str(self.counter_datavi), path=self.path)
    
    def OnHelpIndex(self, event):
        svcom.HelpView(self)
        
    def OnHelpContacts(self, event):
        svcom.Contacts(self)
    
    def OnActivate(self, event):
        self.SetMenuBar(self.menu)
        event.Skip()
    
    def __init__(self):
        wx.Frame.__init__(self, None, -1, "SVI", style= wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
        self.counter_geomodvi=0
        self.counter_meshvi=0
        self.counter_datavi=0
        self.path=os.path.dirname(sys.argv[0])
        if self.path!="": self.path += "/"
        self.resfolder="resources/"
        menu_help = wx.Menu()
        HIndex = menu_help.Append(-1, '&Help Index', 'Help of this applicantion...')
        HContacts = menu_help.Append(-1, '&Contacts', 'Contacts')
        self.menu = wx.MenuBar()
        self.menu.Append(menu_help, '&Help')
        self.SetMenuBar(self.menu)
        box = wx.BoxSizer(wx.VERTICAL)
        btn_splashvi=wx.BitmapButton(self, -1, wx.Image(self.path+self.resfolder+"sismovi.gif", wx.BITMAP_TYPE_GIF).ConvertToBitmap())
        btn_geomodvi=wx.BitmapButton(self, -1, wx.Image(self.path+self.resfolder+"geomodvi.gif", wx.BITMAP_TYPE_GIF).ConvertToBitmap())
        btn_meshvi=wx.BitmapButton(self, -1, wx.Image(self.path+self.resfolder+"meshvi.gif", wx.BITMAP_TYPE_GIF).ConvertToBitmap())
        btn_datavi=wx.BitmapButton(self, -1, wx.Image(self.path+self.resfolder+"datavi.gif", wx.BITMAP_TYPE_GIF).ConvertToBitmap())
        btn_helpvi=wx.BitmapButton(self, -1, wx.Image(self.path+self.resfolder+"helpvi.jpg", wx.BITMAP_TYPE_JPEG).ConvertToBitmap())
        box.Add(btn_splashvi, 0, wx.CENTER)
        box.Add(btn_geomodvi, 0, wx.CENTER)
        box.Add(btn_meshvi, 0, wx.CENTER)
        box.Add(btn_datavi, 0, wx.CENTER)
        box.Add(btn_helpvi, 0, wx.CENTER)
        self.SetSizer(box)
        self.SetAutoLayout(True)
        box.Fit(self)
        self.Show(True)
        self.Bind(wx.EVT_BUTTON, self.OnSplashVi, btn_splashvi)
        self.Bind(wx.EVT_BUTTON, self.OnGeomdVi, btn_geomodvi)
        self.Bind(wx.EVT_BUTTON, self.OnMeshVi, btn_meshvi)
        self.Bind(wx.EVT_BUTTON, self.OnDataVi, btn_datavi)
        self.Bind(wx.EVT_BUTTON, self.OnHelpIndex, btn_helpvi)
        self.Bind(wx.EVT_MENU, self.OnHelpIndex, HIndex)
        self.Bind(wx.EVT_MENU, self.OnHelpContacts, HContacts)
        self.Bind(wx.EVT_ACTIVATE, self.OnActivate)

def main():
    app=wx.PySimpleApp()
    svi=SismoVi()
    app.SetTopWindow(svi)
    app.MainLoop()

if __name__=='__main__':
    main()

