###### commonvi.py

import wx
import wx.lib.plot as plot
import numarray as num, string
import wx.lib.hyperlink as hl
import sys
import os
import wx.html

def HelpView(parent):
    path=os.path.dirname(sys.argv[0])
    if path!="": path += "/"
    resfolder="resources/"
    wx.FileSystem.AddHandler(wx.ZipFSHandler())
    parent.helpctrl = wx.html.HtmlHelpController()
    parent.helpctrl.AddBook(path+resfolder+"Help.htb", 1)
    parent.helpctrl.DisplayContents()
 
def Contacts(parent):
    frame=wx.Frame(parent, -1, "SismoVi contacts!", size=(300,200))
    frame.CentreOnParent()
    label1 = wx.StaticText(frame,-1,"Feedback = ")
    hyper1 = hl.HyperLinkCtrl(frame, wx.ID_ANY, "SismoVi@ogs.trieste.it", URL="mailto:SismoVi@ogs.trieste.it")
    label2 = wx.StaticText(frame,-1,"Web site = ")
    hyper2 = hl.HyperLinkCtrl(frame, wx.ID_ANY, "http://SPICE-rtn.org/SismoVi", URL="http://SPICE-rtn.org/SismoVi")
    sizer=wx.FlexGridSizer(0,2,0,0)
    sizer.Add(label1, 0, wx.ALIGN_LEFT)
    sizer.Add(hyper1, 0, wx.ALIGN_LEFT)
    sizer.Add(label2, 0, wx.ALIGN_LEFT)
    sizer.Add(hyper2, 0, wx.ALIGN_LEFT)
    frame.SetSizer(sizer)
    frame.Show(True)

class PlotFrame(wx.Frame):
    def __init__(self, parent, pos=(0,0), size=(-1,-1), counter=""):
        wx.Frame.__init__(self, parent, -1, "DataVi"+counter+" - plot windows",  pos, size)

        self.Bind(wx.EVT_ACTIVATE, self.OnWindowActivate)

        self.client = plot.PlotCanvas(self)
        self.client.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)
        self.parent=parent

        self.__Layout()

        self.CreateStatusBar(True)
        
        self.Show(True)
        
    def __Layout(self):
        menu_file = wx.Menu()
        FExport = menu_file.Append(-1, 'Export as image...', 'Export the visualization in varius graphic formats')
        FPageSetup = menu_file.Append(-1, 'Page Setup...', 'Setup the printer page')
        FPrintPreview = menu_file.Append(-1, 'Print Preview...', 'Show the current plot on page')
        FPrint = menu_file.Append(-1, 'Print...', 'Print the current plot')
        menu_plot = wx.Menu()
        PZoom = menu_plot.Append(-1, 'Enable &Zoom', 'Enable Mouse Zoom', kind=wx.ITEM_CHECK)
        PGrid = menu_plot.Append(-1, 'Enable &Grid', 'Turn on Grid', kind=wx.ITEM_CHECK)
        menu_help = wx.Menu()
        self.menu = wx.MenuBar()
        self.menu.Append(menu_file, '&File')
        self.menu.Append(menu_plot, '&Plot')

        self.Bind(wx.EVT_MENU, self.OnFileExport, FExport)
        self.Bind(wx.EVT_MENU, self.OnFilePageSetup, FPageSetup)
        self.Bind(wx.EVT_MENU, self.OnFilePrintPreview, FPrintPreview)
        self.Bind(wx.EVT_MENU, self.OnFilePrint, FPrint)
        self.Bind(wx.EVT_MENU, self.OnPlotEnableZoom, PZoom)
        self.Bind(wx.EVT_MENU, self.OnPlotEnableGrid, PGrid)

    def SetParamsDialog(self, frame):
        self.params=frame

    def OnWindowActivate(self, event):
        self.SetMenuBar(self.menu)
        event.Skip()

    def OnMouseLeftDown(self,event):
        s= "Left Mouse Down at Point: (%.4f, %.4f)" % self.client.GetXY(event)
        self.SetStatusText(s)
        event.Skip()

    def OnFilePageSetup(self, event):
        self.client.PageSetup()

    def OnFilePrintPreview(self, event):
        self.client.PrintPreview()

    def OnFilePrint(self, event):
        self.client.Printout()

    def OnFileExport(self, event):
        self.client.SaveFile()

    def OnPlotEnableZoom(self, event):
        self.client.SetEnableZoom(event.IsChecked())

    def OnPlotEnableGrid(self, event):
        self.client.SetEnableGrid(event.IsChecked())

class PolyPolygon(plot.PolyPoints):
    """Class to define line type and style
        - All methods except __init__ are private.
    """

    _attributes = {'colour': 'black',
                   'width': 1,
                   'style': wx.SOLID,
                   'fillcolour': 'black',
                   'fillstyle': wx.SOLID,
                   'legend': ''}

    def __init__(self, points, **attr):
        """Creates PolyLine object
            points - sequence (array, tuple or list) of (x,y) points making up line
            **attr - key word attributes
                Defaults:
                    'colour'= 'black',          - wx.Pen Colour any wxNamedColour
                    'width'= 1,                 - Pen width
                    'style'= wx.SOLID,           - wx.Pen style
                    'legend'= ''                - Line Legend to display
        """
        plot.PolyPoints.__init__(self, points, attr)

    def draw(self, dc, printerScale, coord= None):
        colour = self.attributes['colour']
        width = self.attributes['width'] * printerScale
        style= self.attributes['style']
        fillcolour= self.attributes['fillcolour']
        fillstyle= self.attributes['fillstyle']
        #dc.SetPen(wx.Pen(wx.NamedColour(colour), int(width), style))
        dc.SetPen(wx.Pen(colour, int(width), style))
        if coord == None:
            #dc.SetBrush(wx.Brush(wx.NamedColour(fillcolour),fillstyle))
            dc.SetBrush(wx.Brush(fillcolour,fillstyle))
            dc.DrawPolygon(self.scaled)
        else:
            dc.DrawPolygon(coord) #draw legend line

    def getSymExtent(self, printerScale):
        """Width and Height of Marker"""
        h= self.attributes['width'] * printerScale
        w= 5 * h
        return (w,h)

class LinearMapping:
    def __init__(self,nsteps=0):
        self.SetSteps(nsteps)

    def SetSteps(self,nsteps):
        x=2.*num.arange(nsteps+2)/(nsteps+1)-1
        xp,xm=1+x,1-x
        self.Xpp=num.multiply.outer(xp,xp)*0.25
        self.Xmp=num.multiply.outer(xm,xp)*0.25
        self.Xpm=num.multiply.outer(xp,xm)*0.25
        self.Xmm=num.multiply.outer(xm,xm)*0.25
        self.nsteps=nsteps

    def GetElement(self,a):
        x=a[0,0]*self.Xmm+a[1,0]*self.Xmp+a[2,0]*self.Xpp+a[3,0]*self.Xpm
        y=a[0,1]*self.Xmm+a[1,1]*self.Xmp+a[2,1]*self.Xpp+a[3,1]*self.Xpm
        return num.concatenate((num.reshape(x,(self.nsteps+2,self.nsteps+2,1)),num.reshape(y,(self.nsteps+2,self.nsteps+2,1))),2)
        
    def __call__(self,nodes,elements):
        nodescount=0
        tablenodesref={}
        tableelementsref={}
        newnodes=[]
        newelements=[]
        v=range(4)
        l=range(4)
        for k in elements:
            tt=self.GetElement(nodes[k[:4]-1])
            v[0]=tt[0,0].tolist()
            v[1]=tt[0,self.nsteps+1].tolist()
            v[2]=tt[self.nsteps+1,self.nsteps+1].tolist()
            v[3]=tt[self.nsteps+1,0].tolist()
            l[0]=tt[0,1:self.nsteps+1].tolist()
            l[1]=tt[1:self.nsteps+1,self.nsteps+1].tolist()
            l[2]=tt[self.nsteps+1,self.nsteps:0:-1].tolist()
            l[3]=tt[self.nsteps:0:-1,0].tolist()
            if self.nsteps>0: c=num.reshape(tt[1:self.nsteps+1,1:self.nsteps+1],(self.nsteps*self.nsteps,2)).tolist()
            else: c=[]
            newel=[]
            for j in range(4):
                if tablenodesref.get('v'+str(k[j])):
                    newel+=[tablenodesref.get('v'+str(k[j]))]
                else:
                    newnodes.append(v[j])
                    nodescount+=1
                    tablenodesref['v'+str(k[j])]=nodescount
                    newel+=[nodescount]
            for j in range(4):
                if tableelementsref.get('v'+str(k[j])+'v'+str(k[(j+1)%4])):
                    newel+=tableelementsref['v'+str(k[j])+'v'+str(k[(j+1)%4])]
                elif tableelementsref.get('v'+str(k[(j+1)%4])+'v'+str(k[j])):
                    newel+=tableelementsref['v'+str(k[(j+1)%4])+'v'+str(k[j])][::-1]
                else:
                    newnodes+=l[j]
                    nodescount+=self.nsteps
                    tableelementsref['v'+str(k[j])+'v'+str(k[(j+1)%4])]=range(nodescount-self.nsteps+1,nodescount+1)
                    newel+=range(nodescount-self.nsteps+1,nodescount+1)
            newnodes+=c
            newel+=range(nodescount+1,nodescount+1+self.nsteps**2)+[k[-1]]
            nodescount+=self.nsteps**2
            newelements.append(newel)
            if self.nsteps: iilab='i'
            else: iilab=''
        return 'q1.1'+iilab,num.array(newnodes),num.array(newelements)

class Bl2dRead:
    def __ClearString(self,a):
        while a.count('  '):
            a=string.replace(a,'  ',' ')
        return string.strip(string.replace(string.replace(a,'\r',''),'\n',''))
    
    def __StringToList(self,a):
        return eval('['+string.replace(a,' ',',')+']')

    def __call__(self,filename):
        f=open(filename,'r')
        
        element_type = self.__ClearString(f.readline())
        [nnodes,nelements,ndomains] = self.__StringToList(self.__ClearString(f.readline()))
        
        nodes=[]
        for j in range(nnodes):
            nodes.append(self.__StringToList(self.__ClearString(f.readline())))
        elements=[]
        for j in range(nelements):
            a=self.__StringToList(self.__ClearString(f.readline()))
            if a[-1]: elements.append(a)
        nelements=len(elements)
        
        f.close()
        
        return element_type,num.array(nodes),num.array(elements)

class Bl2dWrite:
    def __call__(self,filename, element_type, nodes, elements):
        f=open(filename,'w')
        
        f.write(str(element_type)+"\n")
        f.write(str(len(nodes))+" "+str(len(elements))+" "+str(elements[:,-1].max())+"\n")
        for j in nodes:
            f.write(str(j)[1:-1]+"\n")
        for j in elements:
            f.write(str(j)[1:-1]+"\n")
        
        f.close()


        
