#! /usr/bin/env python
#
# SismoVi, version 2.00, December 2005.
#      
# Copyright (C) 2004-2005, Francesco Apostolico, Geza Seriani
# This program is free software you can redistribute it and/or 
#     modify it under the terms of the GNU General Public License 
#     as published by the Free Software Foundation either version 2
#     of the License, or any later version, with the following conditions
#     attached in addition to any and all conditions of the GNU
#     General Public License:
#     When reporting or displaying any results or animations created
#     using this code or modification of this code, make the appropriate
#     citation referencing SismoVi by name and including the version
#     number.
#
# This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY without even the implied warranty 
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#     along with this program if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
#     02111-1307  USA
#
# Authors:
#
# Francesco Apostolico, Geza Seriani
# Istituto Nazionale di Oceanografia e di Geofisica Sperimentale - OGS
# Borgo Grotta Gigante, 42/C, Sgonico
# 34010, Trieste, Italy
#
# fapostolico@inogs.it, gseriani@inogs.it
# 
# #######################################/
# SismoVi is a tool for the pre and post processing of data that are used in
#      or generated by seismic modeling.
#      It is platform independent and it is based on 
#      Python 2.3 (http://www.python.org/),
#      numarray 1.3 (http://www.stsci.edu/resources/software_hardware/numarray) and
#      WxPython 2.6 (http://wxpython.org/)
#      that must be installed in your platform before running it. 
#      An interactive help is available when running SismoVi.
#---------------------------------------------------------------------------
###### datavi.py

import wx
import wx.grid
import wx.py
import wx.lib.scrolledpanel as scrolled
import wx.tools.helpviewer as helpv
import wxmpl
import string
import commonvi as svcom
import sismovilib as svlib
import userlib
import numarray as num
import matplotlib as mpl
import matplotlib.cm as cm
import pylab
import os
import sys

class TracesetInteractor:
    def __init__(self, grid, updateMetod=None):
        self._grid=grid
        self._updatem=updateMetod
        
    def _SetTraces(self, n):
        self._grid.DeleteRows(0,self._grid.GetNumberRows())
        self._grid.AppendRows(n)
        for i in range(n):
            self._grid.SetRowLabelValue(i,str(i))
    
    def _SetSamples(self, n):
        self._grid.DeleteCols(0,self._grid.GetNumberCols())
        self._grid.AppendCols(n)
        for i in range(n):
            self._grid.SetColLabelValue(i,str(i))
    
    def value(self, trace, sample):
        return self._data[trace,sample]

    def putvalue(self, trace, sample, value):
        self._data[trace,sample]=value
        self._grid.SetCellValue(trace,sample, str(value))

    def sample(self,sample):
        return self._data[:,sample]

    def putsample(self, sample, values):
        self._data[:,sample]=values
        for j in range(len(values)):
            self._grid.SetCellValue(j,sample, str(values[j]))

    def trace(self,trace):
        return self._data[trace,:]

    def puttrace(self, trace, values):
        self._data[trace,:]=values
        for j in range(len(values)):
            self._grid.SetCellValue(trace,j, str(values[j]))

    def take(self):
        return self._data

    def put(self, data, updatemask=True):
        self._data=data
        self._SetTraces(len(data[:,0]))
        self._SetSamples(len(data[0,:]))
        for j in range(len(data[:,0])):
            for i in range(len(data[0,:])):
                self._grid.SetCellValue(j,i,str(data[j,i]))
        if updatemask: self._updatem(data)

class DataParamsDialog(wx.Frame):
    def __init__(self, parent=None, pos=(0,0), size=(-1,-1), counter="", path=None):
        wx.Frame.__init__(self, parent, -1, "DataVi"+counter+" - input parameters", pos=pos,
            style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
        self.parent=parent
        self.pos=pos
        self.size=size
        self.counter=counter
        if path==None:
            self.path=os.path.dirname(sys.argv[0])
            if self.path!="": self.path += "/"
        else:
            self.path=path
        self.values={}

        self.__Layout()
        
        self.Show(True)
        
######  metods settings for data manipolations
        
        self.text_transf.shell.interp.locals['Sismo'] = self.grid_interact = TracesetInteractor(self.grid_data, self.SetData)
        #self.text_transf.shell.interp.locals['n']=num
        #self.text_transf.shell.interp.locals['m']=mpl
        #self.text_transf.shell.interp.locals['p']=pylab
        #self.text_transf.shell.interp.locals['s']=svlib
        #self.text_transf.shell.interp.locals['u']=userlib

    def __Layout(self):

###### list of objects in frame

# make tabs
        self.tabs=wx.Notebook(self, -1)

# make radio
        self.radio_format = wx.RadioBox(self, -1, "",
             choices=["Wiggle", "Variable Area", "Wiggle+V.A.", "Density", "Contour", "D. Contour"],
             majorDimension=0, style=wx.RA_SPECIFY_COLS)

# make buttons
        self.button_draw = wx.Button(self, -1, "Visualize")
        self.button_reset = wx.Button(self, -1, "Reset")
        self.button_hidden = wx.Button(self, -1, "Hidden")

# make scrolled-panels
        self.panel_data=scrolled.ScrolledPanel(self.tabs)
        self.panel_view=scrolled.ScrolledPanel(self.tabs)
        self.panel_transf=scrolled.ScrolledPanel(self.tabs)

# make label and text for data_panel
        self.label_plot_title = wx.StaticText(self.panel_data, -1, "Title:")
        self.label_trace_label = wx.StaticText(self.panel_data, -1, "Trace label:")
        self.label_sample_label = wx.StaticText(self.panel_data, -1, "Sample label:")
        self.label_traces = wx.StaticText(self.panel_data, -1, "Traces:")
        self.label_trace_interval = wx.StaticText(self.panel_data, -1, "Trace interval:")
        self.label_samples = wx.StaticText(self.panel_data, -1, "Samples:")
        self.label_sample_interval = wx.StaticText(self.panel_data, -1, "Sampling interval:")
        self.label_peak = wx.StaticText(self.panel_data, -1, "Peak value:")
        self.label_average = wx.StaticText(self.panel_data, -1, "Average value:")
        self.text_plot_title = wx.TextCtrl(self.panel_data, -1, size=(333,-1))
        self.text_trace_label = wx.TextCtrl(self.panel_data, -1)
        self.text_sample_label = wx.TextCtrl(self.panel_data, -1)
        self.text_traces = wx.TextCtrl(self.panel_data, -1)
        self.text_traces.SetForegroundColour((0,0,255))
        self.text_traces.SetEditable(False)
        self.text_trace_interval = wx.TextCtrl(self.panel_data, -1)
        self.text_samples = wx.TextCtrl(self.panel_data, -1)
        self.text_samples.SetForegroundColour((0,0,255))
        self.text_samples.SetEditable(False)
        self.text_sample_interval = wx.TextCtrl(self.panel_data, -1)
        self.text_peak = wx.TextCtrl(self.panel_data, -1)
        self.text_peak.SetForegroundColour((0,0,255))
        self.text_peak.SetEditable(False)
        self.text_average = wx.TextCtrl(self.panel_data, -1)
        self.text_average.SetForegroundColour((0,0,255))
        self.text_average.SetEditable(False)
    
# make grid
        self.grid_data = wx.grid.Grid(self.panel_data, -1, size=(650,300))
        self.grid_data.CreateGrid(20,10)

# make label and text for view_panel
        self.label_sample_params = wx.StaticText(self.panel_view, -1, "Sample parameters")
        font=self.label_sample_params.GetFont()
        font.SetWeight(wx.BOLD)
        self.label_sample_params.SetFont(font)
        self.label_trace_params = wx.StaticText(self.panel_view, -1, "Trace parameters")
        font=self.label_trace_params.GetFont()
        font.SetWeight(wx.BOLD)
        self.label_trace_params.SetFont(font)
        self.label_colour_params = wx.StaticText(self.panel_view, -1, "Colour parameters")
        font=self.label_colour_params.GetFont()
        font.SetWeight(wx.BOLD)
        self.label_colour_params.SetFont(font)
        self.label_sample_first = wx.StaticText(self.panel_view, -1, "First:")
        self.label_sample_last = wx.StaticText(self.panel_view, -1, "Last:")
        self.label_sample_dec = wx.StaticText(self.panel_view, -1, "Decimation factor:")
        self.label_sample_origin = wx.StaticText(self.panel_view, -1, "Origin:")
        self.label_sample_length = wx.StaticText(self.panel_view, -1, "Window length:")
        self.label_sample_rescale = wx.StaticText(self.panel_view, -1, "Tick scale:")
        self.label_trace_first = wx.StaticText(self.panel_view, -1, "First:")
        self.label_trace_last = wx.StaticText(self.panel_view, -1, "Last:")
        self.label_trace_dec = wx.StaticText(self.panel_view, -1, "Decimation factor:")
        self.label_trace_clip = wx.StaticText(self.panel_view, -1, "Clip factor (%):")
        self.label_trace_overlap = wx.StaticText(self.panel_view, -1, "Overlap factor (%):")
        self.label_trace_origin = wx.StaticText(self.panel_view, -1, "Origin:")
        self.label_trace_length = wx.StaticText(self.panel_view, -1, "Window length:")
        self.label_trace_rescale = wx.StaticText(self.panel_view, -1, "Tick scale:")
        self.label_colour_mmm = wx.StaticText(self.panel_view, -1, "Min/Med/Max:")
        self.spin_sample_first = wx.SpinCtrl(self.panel_view, -1)
        self.spin_sample_last = wx.SpinCtrl(self.panel_view, -1)
        self.spin_sample_dec = wx.SpinCtrl(self.panel_view, -1)
        self.text_sample_origin = wx.TextCtrl(self.panel_view, -1)
        self.text_sample_length = wx.TextCtrl(self.panel_view, -1)
        self.text_sample_rescale = wx.TextCtrl(self.panel_view, -1)
        self.spin_trace_first = wx.SpinCtrl(self.panel_view, -1)
        self.spin_trace_last = wx.SpinCtrl(self.panel_view, -1)
        self.spin_trace_dec = wx.SpinCtrl(self.panel_view, -1)
        self.text_trace_clip = wx.TextCtrl(self.panel_view, -1)
        self.text_trace_overlap = wx.TextCtrl(self.panel_view, -1)
        self.text_trace_origin = wx.TextCtrl(self.panel_view, -1)
        self.text_trace_length = wx.TextCtrl(self.panel_view, -1)
        self.text_trace_rescale = wx.TextCtrl(self.panel_view, -1)
        self.select_colour = wx.ComboBox(self.panel_view, -1, "jet",\
            choices=['bone','cool','copper','flag','gray','hot','hsv','jet','pink','prism','spring','summer','winter'])

# make label and text for transf_panel
        self.panel_crust=wx.Panel(self.panel_transf,-1)
        self.text_transf = wx.py.crust.Crust(self.panel_crust, size=(650,420))
        self.button_revert_transf = wx.Button(self.panel_transf, -1, "Revert to original data")
        self.button_sendinto_transf = wx.Button(self.panel_transf, -1, "Update original data")
        self.button_openscript_transf = wx.Button(self.panel_transf, -1, "Open script")
        self.button_savescript_transf = wx.Button(self.panel_transf, -1, "Save script")

###### layout of objects in frame

# add pages at tabs
        self.tabs.AddPage(self.panel_data, 'Data', True)
        self.tabs.AddPage(self.panel_view, 'Visualization', False)
        self.tabs.AddPage(self.panel_transf, 'Transformation', False)
        
# add sizer for label and text of data_panel
        self.sizer_data_params=wx.GridBagSizer(5,5)
        self.sizer_data_params.Add(self.label_plot_title, (0,0))
        self.sizer_data_params.Add(self.text_plot_title, (0,1), (1,3))
        self.sizer_data_params.Add(self.label_sample_label, (1,0))
        self.sizer_data_params.Add(self.text_sample_label, (1,1))
        self.sizer_data_params.Add(self.label_trace_label, (1,2))
        self.sizer_data_params.Add(self.text_trace_label, (1,3))
        self.sizer_data_params.Add(self.label_samples, (2,0))
        self.sizer_data_params.Add(self.text_samples, (2,1))
        self.sizer_data_params.Add(self.label_traces, (2,2))
        self.sizer_data_params.Add(self.text_traces, (2,3))
        self.sizer_data_params.Add(self.label_sample_interval, (3,0))
        self.sizer_data_params.Add(self.text_sample_interval, (3,1))
        self.sizer_data_params.Add(self.label_trace_interval, (3,2))
        self.sizer_data_params.Add(self.text_trace_interval, (3,3))
        self.sizer_data_params.Add(self.label_peak, (4,0))
        self.sizer_data_params.Add(self.text_peak, (4,1))
        self.sizer_data_params.Add(self.label_average, (4,2))
        self.sizer_data_params.Add(self.text_average, (4,3))

# add main sizer for data_panel
        self.sizer_data_main=wx.FlexGridSizer(0,1,0,0)
        self.sizer_data_main.Add(self.sizer_data_params,0,wx.ALIGN_CENTER)
        self.sizer_data_main.Add((15,15))
        self.sizer_data_main.Add(self.grid_data,0,wx.ALIGN_CENTER)
        self.panel_data.SetSizer(self.sizer_data_main)

# add sizer for view
        self.sizer_view=wx.GridSizer(0,4,0,0)
        self.sizer_view.Add(self.label_sample_params)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_sample_first)
        self.sizer_view.Add(self.spin_sample_first)
        self.sizer_view.Add(self.label_sample_last)
        self.sizer_view.Add(self.spin_sample_last)
        self.sizer_view.Add(self.label_sample_dec)
        self.sizer_view.Add(self.spin_sample_dec)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_sample_origin)
        self.sizer_view.Add(self.text_sample_origin)
        self.sizer_view.Add(self.label_sample_length)
        self.sizer_view.Add(self.text_sample_length)
        self.sizer_view.Add(self.label_sample_rescale)
        self.sizer_view.Add(self.text_sample_rescale)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_trace_params)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_trace_first)
        self.sizer_view.Add(self.spin_trace_first)
        self.sizer_view.Add(self.label_trace_last)
        self.sizer_view.Add(self.spin_trace_last)
        self.sizer_view.Add(self.label_trace_dec)
        self.sizer_view.Add(self.spin_trace_dec)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_trace_origin)
        self.sizer_view.Add(self.text_trace_origin)
        self.sizer_view.Add(self.label_trace_length)
        self.sizer_view.Add(self.text_trace_length)
        self.sizer_view.Add(self.label_trace_rescale)
        self.sizer_view.Add(self.text_trace_rescale)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_trace_clip)
        self.sizer_view.Add(self.text_trace_clip)
        self.sizer_view.Add(self.label_trace_overlap)
        self.sizer_view.Add(self.text_trace_overlap)
        self.sizer_view.Add(self.label_colour_params)
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add((1,1))
        self.sizer_view.Add(self.label_colour_mmm)
        self.sizer_view.Add(self.select_colour)

        self.panel_view.SetSizer(self.sizer_view)

# add sizer for buttons of transf
        self.sizer_transfleftbutton=wx.FlexGridSizer(0,4,5,10)
        self.sizer_transfleftbutton.Add(self.button_revert_transf,0,wx.ALIGN_CENTER)
        self.sizer_transfleftbutton.Add(self.button_sendinto_transf,0,wx.ALIGN_CENTER)
        self.sizer_transfleftbutton.Add(self.button_openscript_transf,0,wx.ALIGN_CENTER)
        self.sizer_transfleftbutton.Add(self.button_savescript_transf,0,wx.ALIGN_CENTER)

# add sizer for transf
        self.sizer_transf=wx.FlexGridSizer(0,1,10,10)
        self.sizer_transf.Add(self.panel_crust,0,wx.ALIGN_CENTER)
        self.sizer_transf.Add(self.sizer_transfleftbutton,0,wx.ALIGN_CENTER)
        self.panel_transf.SetSizer(self.sizer_transf)

# add button sizer
        self.sizer_buttons=wx.GridSizer(0,3,10,10)
        self.sizer_buttons.Add(self.button_draw)
        self.sizer_buttons.Add(self.button_reset)
        self.sizer_buttons.Add(self.button_hidden)
        
# add main sizer        
        self.sizer_main=wx.FlexGridSizer(0,1,0,0)
        self.sizer_main.Add(self.tabs, 0, wx.ALIGN_CENTER)
        self.sizer_main.Add((15,15))
        self.sizer_main.Add(self.radio_format, 0, wx.ALIGN_CENTER)
        self.sizer_main.Add((15,15))
        self.sizer_main.Add(self.sizer_buttons, 0, wx.ALIGN_CENTER)                
        self.sizer_main.Add((15,15))
        self.SetSizer(self.sizer_main)

        self.SetAutoLayout(True)
        
        self.panel_data.Fit()
        self.panel_view.Fit()
        self.panel_transf.Fit()
        self.sizer_main.Fit(self)

###### make manu
        menu_file = wx.Menu()
        FOpen = menu_file.Append(-1, 'Open Data...', 'Open file for plotting')
        FSave = menu_file.Append(-1, 'Save Data...', 'Save file for plotting')
        FExit = menu_file.Append(-1, 'E&xit', 'Quit from DataVi')
        menu_help = wx.Menu()
        HIndex = menu_help.Append(-1, '&Help Index', 'Help of this applicantion...')
        HContacts = menu_help.Append(-1, '&Contacts', 'Contacts')
        self.menu = wx.MenuBar()
        self.menu.Append(menu_file, '&File')
        self.menu.Append(menu_help, '&Help')
        
        FSave.Enable(False)

###### make events of objects in frame
        self.Bind(wx.EVT_ACTIVATE, self.OnWindowActivate)
        self.Bind(wx.EVT_BUTTON, self.OnWindowSubmit, self.button_draw)
        self.Bind(wx.EVT_BUTTON, self.OnWindowReset, self.button_reset)
        self.Bind(wx.EVT_BUTTON, self.OnTransfRevert, self.button_revert_transf)
        self.Bind(wx.EVT_BUTTON, self.OnTransfSendInto, self.button_sendinto_transf)
        self.Bind(wx.EVT_BUTTON, self.OnTransfOpen, self.button_openscript_transf)
        self.Bind(wx.EVT_BUTTON, self.OnTransfSave, self.button_savescript_transf)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.OnCellChange, self.grid_data)
        self.Bind(wx.EVT_MENU, self.OnFileOpen, FOpen)
        self.Bind(wx.EVT_MENU, self.OnFileSave, FSave)
        self.Bind(wx.EVT_MENU, self.OnFileExit, FExit)
        self.Bind(wx.EVT_MENU, self.OnHelpIndex, HIndex)
        self.Bind(wx.EVT_MENU, self.OnHelpContacts, HContacts)

    def SetParamsDialog(self, frame):
        self.params=frame

    def OnFileExit(self, event):
        self.Close()

    def OnFileOpen(self, event):
        dlg = wx.FileDialog(self, "Open File...", self.path, style=wx.OPEN, wildcard = "DataVi files (*.sismo)|*.sismo|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            fname = dlg.GetPath()
            f=open(fname,"r")
            self.Original=eval(f.read())
            f.close()
            self.SetValues(self.Original.copy())
        dlg.Destroy()

    def OnFileSave(self, event):
        pass
        # not yet implemented

    def OnHelpIndex(self, event):
        svcom.HelpView(self)
        
    def OnHelpContacts(self, event):
        svcom.Contacts(self)

    def OnWindowActivate(self, event):
        self.SetMenuBar(self.menu)
        event.Skip()

    def OnWindowSubmit(self, event):
        self.wxmplRender()

    def OnWindowReset(self, event):
        self.SetMask()

    def OnTransfRevert(self, event):
        self.SetValues(self.Original.copy())

    def OnTransfSendInto(self, event):
        self.Original=self.values.copy()

    def OnTransfOpen(self, event):
        dlg = wx.FileDialog(self, "Open Script file...", self.path, style=wx.OPEN, wildcard = "*")
        if dlg.ShowModal() == wx.ID_OK:
            fname = dlg.GetPath()
            f=open(fname,"r")
            a=f.read()
            f.close()
            b=string.split(a,"\n")
            for i in b:
                self.text_transf.shell.run(i)
        dlg.Destroy()

    def OnTransfSave(self, event):
        dlg = wx.FileDialog(self, "Save Script file...", self.path, style=wx.SAVE, wildcard = "Script files (*.py)|*.pick|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            f=open(dlg.GetPath(),"w")
            f.write(self.text_transf.sessionlisting.GetValue())
            f.close()
        dlg.Destroy()

    def OnCellChange(self, event):
        self.values['data'][event.GetRow(),event.GetCol()]=eval(self.grid_data.GetCellValue(event.GetRow(),event.GetCol()))

    def UpdateDataset(self, data):
        self.values['data']=data.copy()
        
    def SetValues(self, values):
        self.values=values
        self.AdjustValues()
        self.SetMask()

    def SetData(self, data):
        self.values['data']=data
        self.AdjustValues()
        self.SetMask()

    def AdjustValues(self):
        if not(self.values.get('title')):self.values['title']='Plot'
        if not(self.values.get('trace_label')):self.values['trace_label']='Traces'
        if not(self.values.get('sample_label')):self.values['sample_label']='Samples'
        if not(self.values.get('trace_interval')):self.values['trace_interval']=1
        if not(self.values.get('sampling_interval')):self.values['sampling_interval']=1
        if type(self.values.get('data')[0])==list or type(self.values.get('data')[0])==type(num.array(())):
            maxlen=0
            for j in self.values.get('data'):
                if maxlen<len(j): maxlen=len(j)
            data=num.zeros((len(self.values.get('data')),maxlen),'Float')
            for j in range(len(self.values.get('data'))):
                data[j,0:len(self.values.get('data')[j])]=self.values.get('data')[j][0:len(self.values.get('data')[j])]
            self.values['data']=num.array(data)
            (self.values['traces'],self.values['samples'])=self.values['data'].shape
        else:
            self.values['data']=num.array(self.values.get('data'))
            self.values['data'].shape=(self.values['traces'],self.values['samples'])
        if self.values.get('data_order')=='trace':
            self.values['data']=num.transpose(self.values['data'])
        else:
            self.values['data_order']='sample'
        if type(self.values.get('trace_interval'))==int or type(self.values.get('trace_interval'))==float:
            self.values['trace_origin']=0
            self.values['trace_length']=float((self.values.get('traces')-1)*self.values.get('trace_interval'))
        elif type(self.values.get('trace_interval'))==list:
            if len(self.values.get('trace_interval'))<self.values.get('traces'):
                x=num.array(self.values.get('trace_interval'))
                y=x[1:]-x[0]
                d=self.values.get('traces')/len(y)
                for i in range(d+1):
                    x=num.concatenate((x,y+x[-1]))
                self.values['trace_interval']=x[:self.values.get('traces')].tolist()
            elif len(self.values.get('trace_interval'))>self.values.get('traces'):
                self.values['trace_interval']=self.values.get('trace_interval')[:self.values.get('traces')]
            self.values['trace_origin']=self.values.get('trace_interval')[0]
            self.values['trace_length']=self.values.get('trace_interval')[-1]-self.values.get('trace_interval')[0]                
        else:
            self.values['trace_interval']=''
            self.values['trace_origin']=''
            self.values['trace_length']=''
        if type(self.values.get('sampling_interval'))==int or type(self.values.get('sampling_interval'))==float:
            self.values['sampling_origin']=0
            self.values['sampling_length']=float((self.values.get('samples')-1)*self.values.get('sampling_interval'))
        elif type(self.values.get('sampling_interval'))==list:
            if len(self.values.get('sampling_interval'))<self.values.get('samples'):
                x=num.array(self.values.get('sampling_interval'))
                y=x[1:]-x[0]
                d=self.values.get('samples')/(len(self.values.get('sampling_interval'))-1)
                for i in range(d+1):
                    x=num.concatenate((x,y+x[-1]))
                self.values['sampling_interval']=x[:self.values.get('samples')].tolist()
            elif len(self.values.get('sampling_interval'))>self.values.get('smaples'):
                self.values['sampling_interval']=self.values.get('sampling_interval')[:self.values.get('samples')]
            self.values['sampling_origin']=self.values.get('sampling_interval')[0]
            self.values['sampling_length']=self.values.get('sampling_interval')[-1]-self.values.get('sampling_interval')[0]                
        else:
            self.values['sampling_interval']=''
            self.values['sampling_origin']=''
            self.values['sampling_length']=''

    def SetMask(self):
        self.text_plot_title.SetValue(str(self.values.get('title')))
        self.text_trace_label.SetValue(str(self.values.get('trace_label')))
        self.text_sample_label.SetValue(str(self.values.get('sample_label')))
        self.text_traces.SetValue(str(self.values.get('traces')))
        self.text_trace_interval.SetValue(str(self.values.get('trace_interval')))
        self.spin_trace_first.SetValue(0)
        self.spin_trace_first.SetRange(0,self.values.get('traces')-1)
        self.spin_trace_last.SetValue(self.values.get('traces')-1)
        self.spin_trace_last.SetRange(0,self.values.get('traces')-1)
        self.spin_trace_dec.SetValue(1)
        self.spin_trace_dec.SetRange(1,self.values.get('traces'))
        self.text_trace_clip.SetValue('0')
        self.text_trace_overlap.SetValue('0')
        self.text_trace_origin.SetValue(str(self.values['trace_origin']))
        self.text_trace_length.SetValue(str(self.values['trace_length']))
        self.text_trace_rescale.SetValue('1')
        self.text_samples.SetValue(str(self.values.get('samples')))
        self.text_sample_interval.SetValue(str(self.values.get('sampling_interval')))
        self.spin_sample_first.SetValue(0)
        self.spin_sample_first.SetRange(0,self.values.get('samples')-1)
        self.spin_sample_last.SetValue(self.values.get('samples')-1)
        self.spin_sample_last.SetRange(0,self.values.get('samples')-1)
        self.spin_sample_dec.SetValue(1)
        self.spin_sample_dec.SetRange(1,self.values.get('samples'))
        self.text_sample_origin.SetValue(str(self.values['sampling_origin']))
        self.text_sample_length.SetValue(str(self.values['sampling_length']))
        self.text_sample_rescale.SetValue('1')
        self.grid_interact.put(self.values['data'], False)
        data_abs=num.fabs(self.values['data'])
        data_max=data_abs.max()
        data_ave=data_abs.mean()
        self.text_peak.SetValue(str(data_max))
        self.text_average.SetValue(str(data_ave))

    def wxmplRender(self):
        def make_x():
            a=eval(self.text_sample_interval.GetValue())
            if type(a)==list:
                sampleint=num.array(a)
            else:
                sampleint=num.arange(samples)*a
            return sampleint[sample_first:sample_last+1]

        def make_y():
            a=eval(self.text_trace_interval.GetValue())
            if type(a)==list:
                traceint=num.array(a)
            else:
                traceint=num.arange(traces)*a
            return traceint
            
        frame = wxmpl.PlotFrame(self, -1, "Plot", size=(8,5), dpi=96)
        figure = frame.get_figure()
        axes = figure.gca()
        axes.set_xlabel(self.text_sample_label.GetValue())
        axes.set_ylabel(self.text_trace_label.GetValue())
        axes.set_title(self.text_plot_title.GetValue())
        
        data = self.grid_interact.take()
        traces, samples = data.shape
        sample_first = int(self.spin_sample_first.GetValue())
        sample_last = int(self.spin_sample_last.GetValue())
        sample_decimation = int(self.spin_sample_dec.GetValue())
        trace_first = int(self.spin_trace_first.GetValue())
        trace_last = int(self.spin_trace_last.GetValue())
        trace_decimation = int(self.spin_trace_dec.GetValue())
        clipping=float(self.text_trace_clip.GetValue())
        overlap=float(self.text_trace_overlap.GetValue())
        peak=float(self.text_peak.GetValue())
        format = self.radio_format.GetSelection()

        ss=make_x()
        tt=make_y()
        mint=(tt[1:]-tt[:-1]).min()
        data=num.clip(data,-peak*(100-clipping)*0.01,peak*(100-clipping)*0.01)/peak/2*mint/(100-clipping)*trace_decimation/(100-overlap)*10000
        if format<3:
            if format==1:
                data=num.clip(data,0,data.max())
            elif format==2:
                data1=num.clip(data,0,data.max())
            for j in range(trace_first,trace_last,trace_decimation):
                if format==0:
                    axes.plot(ss[sample_first:sample_last:sample_decimation],\
                        tt[j]+data[j,sample_first:sample_last:sample_decimation],'k-')
                elif format==1:
                    axes.fill(ss[sample_first:sample_last:sample_decimation],\
                        tt[j]+data[j,sample_first:sample_last:sample_decimation],'k', linewidth=0, edgecolor="w")
                elif format==2:
                    axes.fill(ss[sample_first:sample_last:sample_decimation],\
                        tt[j]+data1[j,sample_first:sample_last:sample_decimation],'k', linewidth=0, edgecolor="w")
                    axes.plot(ss[sample_first:sample_last:sample_decimation],\
                        tt[j]+data[j,sample_first:sample_last:sample_decimation],'k-')
        else:
            ms,mt=pylab.meshgrid(ss,tt)
            if format==3:
                axes.pcolor(ms,mt,data,shading='flat', cmap=eval("cm."+self.select_colour.GetValue()))
            elif format==4:
                axes.contour(ms,mt,data, cmap=eval("cm."+self.select_colour.GetValue()))
            elif format==5:
                axes.contourf(ms,mt,data, cmap=eval("cm."+self.select_colour.GetValue()))
        frame.draw()
        frame.Show()

def main(*args, **kwds):
    app=wx.PySimpleApp()
    DataParamsDialog(*args, **kwds)
    app.MainLoop()

if __name__=='__main__':
    main()
