#! /usr/bin/env python
#
# SismoVi, version 2.00, December 2005.
#      
# Copyright (C) 2004-2005, Francesco Apostolico, Geza Seriani
# This program is free software you can redistribute it and/or 
#     modify it under the terms of the GNU General Public License 
#     as published by the Free Software Foundation either version 2
#     of the License, or any later version, with the following conditions
#     attached in addition to any and all conditions of the GNU
#     General Public License:
#     When reporting or displaying any results or animations created
#     using this code or modification of this code, make the appropriate
#     citation referencing SismoVi by name and including the version
#     number.
#
# This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY without even the implied warranty 
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#     along with this program if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
#     02111-1307  USA
#
# Authors:
#
# Francesco Apostolico, Geza Seriani
# Istituto Nazionale di Oceanografia e di Geofisica Sperimentale - OGS
# Borgo Grotta Gigante, 42/C, Sgonico
# 34010, Trieste, Italy
#
# fapostolico@inogs.it, gseriani@inogs.it
# 
# #######################################/
# SismoVi is a tool for the pre and post processing of data that are used in
#      or generated by seismic modeling.
#      It is platform independent and it is based on 
#      Python 2.3 (http://www.python.org/),
#      numarray 1.3 (http://www.stsci.edu/resources/software_hardware/numarray) and
#      WxPython 2.6 (http://wxpython.org/)
#      that must be installed in your platform before running it. 
#      An interactive help is available when running SismoVi.
#---------------------------------------------------------------------------
###### geomodvi.py

import wx
import wx.lib.ogl as ogl
import sys
import os
import commonvi as svcom
import numarray as num
import random
import math
import string

#----------------------------------------------------------------------

class GeoText(wx.Dialog):
    def __init__(self, parent, text):
        wx.Dialog.__init__(self, parent, -1, '', size=(420, 380) )
        vi = wx.TextCtrl(self, -1, style=wx.TE_MULTILINE)
        vi.SetValue(text)
        button = wx.Button(self, wx.ID_OK, "Close")
        lc = wx.LayoutConstraints()
        lc.top.SameAs(self, wx.Top, 5)
        lc.left.SameAs(self, wx.Left, 5)
        lc.bottom.SameAs(button, wx.Top, 5)
        lc.right.SameAs(self, wx.Right, 5)
        vi.SetConstraints(lc)
        lc = wx.LayoutConstraints()
        lc.bottom.SameAs(self, wx.Bottom, 5)
        lc.centreX.SameAs(self, wx.CentreX)
        lc.width.AsIs()
        lc.height.AsIs()
        button.SetConstraints(lc)
        self.SetAutoLayout(True)
        self.Layout()
        self.CentreOnParent(wx.BOTH)

#----------------------------------------------------------------------

class GeoInputDialog(wx.Dialog):
    def __init__(self, parent, titolo, etich, predef,\
     pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.DEFAULT_DIALOG_STYLE):
        pre = wx.PreDialog()
        pre.Create(parent, -1, titolo, pos, size, style)
        self.this = pre.this
        sizer = wx.BoxSizer(wx.VERTICAL)
        label = wx.StaticText(self, -1, titolo)
        sizer.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.text=[]
        for k in range(len(etich)):
            box = wx.BoxSizer(wx.HORIZONTAL)
            box.Add(wx.StaticText(self, -1, etich[k]), 0, wx.ALIGN_CENTRE|wx.ALL, 5)
            self.text.append(wx.TextCtrl(self, -1, str(predef[k]), size=(80,-1)))
            box.Add(self.text[-1], 1, wx.ALIGN_CENTRE|wx.ALL, 5)
            sizer.AddSizer(box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
        line = wx.StaticLine(self, -1, size=(20,-1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.RIGHT|wx.TOP, 5)
        box = wx.BoxSizer(wx.HORIZONTAL)
        btn = wx.Button(self, wx.ID_OK, " OK ")
        btn.SetDefault()
        box.Add(btn, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        btn = wx.Button(self, wx.ID_CANCEL, " Cancel ")
        box.Add(btn, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        sizer.AddSizer(box, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def Destroy(self):
        x=[]
        for k in range(len(self.text)):
            x.append(self.text[k].GetValue())
        return x

#----------------------------------------------------------------------

class GeoParDialog(wx.Dialog):
    def __init__(self, parent, formula):
        # begin wxGlade: MyDialog2.__init__
        #kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, parent, -1, "", style=wx.DEFAULT_DIALOG_STYLE)
        self.label_1 = wx.StaticText(self, -1, "Grid step")
        self.radio_box_1 = wx.RadioBox(self, -1, "Formula:", choices=["vel/10*6/5.2", "vel/5*6/5.2", "vel/1*6/5.2"], majorDimension=0, style=wx.RA_SPECIFY_ROWS)
        self.text_ctrl_1 = wx.TextCtrl(self, -1, "")
        self.button_1 = wx.Button(self, wx.ID_OK, "Submit")

        self.__set_properties(formula)
        self.__do_layout()
        # end wxGlade
        wx.EVT_RADIOBOX(self, self.radio_box_1.GetId(), self.OnChange)
    
    def OnChange(self, event):
        self.text_ctrl_1.SetValue(self.radio_box_1.GetItemLabel(self.radio_box_1.GetSelection()))

    def __set_properties(self, formula):
        # begin wxGlade: MyDialog2.__set_properties
        self.SetTitle("Insert formula!")
        for i in range(self.radio_box_1.GetCount()):
            if self.radio_box_1.GetItemLabel(i)==formula:
                self.radio_box_1.SetSelection(i)
        # end wxGlade
        self.text_ctrl_1.SetValue(formula)

    def __do_layout(self):
        # begin wxGlade: MyDialog2.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.label_1, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_1.Add(self.radio_box_1, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_1.Add(self.text_ctrl_1, 0, wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_1.Add(self.button_1, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)
        self.SetAutoLayout(1)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        sizer_1.SetSizeHints(self)
        self.Layout()
        # end wxGlade

    def Destroy(self):
        wx.Dialog.Destroy(self)
        return self.text_ctrl_1.GetValue()

#----------------------------------------------------------------------

class GeoEvent(ogl.ShapeEvtHandler):

    def __init__(self, frame=None):
        ogl.ShapeEvtHandler.__init__(self)
        self.frame=frame
        self.drag=False

    def SetShape(self, shape):
        ogl.ShapeEvtHandler.SetShape(self, shape)
        self.shape=self.GetShape()
        self.canvas=self.shape.GetCanvas()
        self.diagram=self.canvas.GetDiagram()

    def OnLeftClick(self, x, y, keys = 0, attachment = 0):
        self.dc=wx.PaintDC(self.canvas)
        self.canvas.PrepareDC(self.dc)
        if self.GetShape().tipo=="Background":
            if self.frame.Toolbar.GetToolState(self.frame.T121): self.SetCoord(x, y)
            elif self.frame.Toolbar.GetToolState(self.frame.T130): self.InsNodo(x, y)
            elif self.frame.Toolbar.GetToolState(self.frame.T140): self.InsLinea(x, y)
            elif self.frame.Toolbar.GetToolState(self.frame.T141): self.InsPoligonale(x, y)
            elif self.frame.Toolbar.GetToolState(self.frame.T172): self.SetGrid(x, y)
        elif self.GetShape().tipo=="Nodo":
            if self.frame.Toolbar.GetToolState(self.frame.T131): self.DelShape()
            elif self.frame.Toolbar.GetToolState(self.frame.T140): self.InsLinea(x, y)
            elif self.frame.Toolbar.GetToolState(self.frame.T141): self.InsPoligonale(x, y)
            elif self.frame.Toolbar.GetToolState(self.frame.T150): self.InsDominio(-1)
            elif self.frame.Toolbar.GetToolState(self.frame.T151): self.InsDominio(1)
            elif self.frame.Toolbar.GetToolState(self.frame.T161): self.SetVelocita()
            elif self.frame.Toolbar.GetToolState(self.frame.T172): self.SetGrid(x, y)
            else: self.SetStatusbar(self.shape)
        elif self.GetShape().tipo=="Linea":
            if self.frame.Toolbar.GetToolState(self.frame.T131): self.DelShape()
        elif self.GetShape().tipo=="Dominio":
            if self.frame.Toolbar.GetToolState(self.frame.T131): self.DelShape()
            elif self.frame.Toolbar.GetToolState(self.frame.T161): self.SetVelocita()
            else: self.SetStatusbar(self.shape)

    def OnEndDragLeft(self, x, y, keys=0, attachment=0):
        shape = self.GetShape()
        ogl.ShapeEvtHandler.OnEndDragLeft(self, x, y, keys, attachment)

        if not shape.Selected():
            self.OnLeftClick(x, y, keys, attachment)
        self.shape.x=x/self.frame.Zoom
        self.shape.y=y/self.frame.Zoom
        for ss in self.diagram.GetShapeList():
            if ss.tipo=="Dominio":
                if ss.nodi.count(self.shape):
                    lnn=[]
                    for nn in ss.nodi:
                        lnn.append((nn.GetX(),nn.GetY()))
                        ss.Create(lnn)
        self.canvas.Redraw(self.dc)

    def OnSizingEndDragLeft(self, pt, x, y, keys, attch):
        ogl.ShapeEvtHandler.OnSizingEndDragLeft(self, pt, x, y, keys, attch)

    def OnMovePost(self, dc, x, y, oldX, oldY, display):
        ogl.ShapeEvtHandler.OnMovePost(self, dc, x, y, oldX, oldY, display)

    def OnRightClick(self, *dontcare):
        self.log.WriteText("%s\n" % self.GetShape())


    def SetCoord(self, x, y):
        if len(self.frame.w)>=0 and len(self.frame.w)<=3:
            nodo=self.frame.AddAssi(x, y)
            self.canvas.Redraw(self.dc)
            if len(self.frame.w)>=0 and len(self.frame.w)<=2:
                errore=True
                while(errore):
                    inp=GeoInputDialog(self.frame, "Point "+str(len(self.frame.w)+1), ["X:","Y:"], [nodo.GetX()/self.frame.Zoom,nodo.GetY()/self.frame.Zoom])
                    val=inp.ShowModal()
                    res=inp.Destroy()
                    if val==wx.ID_OK:
                        try:
                            x=float(res[0])
                            y=float(res[1])
                            errore=False
                            self.frame.w.append(x)
                            self.frame.z.append(y)
                        except:
                            msg=wx.MessageDialog(self.frame, "Invalid values!", style=wx.OK)
                            msg.ShowModal()
                            msg.Destroy()
                            errore=True
                    else:
                        msg=wx.MessageDialog(self.frame, "Operation canceled!", style=wx.OK)
                        msg.ShowModal()
                        msg.Destroy()
                        for nodo in self.diagram.GetShapeList():
                            if nodo.tipo=="Assi":
                                self.diagram.RemoveShape(nodo)
                                #nodo.Destroy()
                                self.Redraw(self.dc)
                        self.frame.w=[]
                        self.frame.z=[]
                        errore=False
            else:
                x=[]
                y=[]
                for ss in self.diagram.GetShapeList():
                    if ss.tipo=="Assi":
                        x.append(ss.GetX()/self.frame.Zoom)
                        y.append(ss.GetY()/self.frame.Zoom)
                w=self.frame.w
                z=self.frame.z
                delta=float(x[0]*y[1]-x[1]*y[0]+x[1]*y[2]-x[2]*y[1]+x[2]*y[0]-x[0]*y[2])
                parX=[]
                parX.append((w[0]*y[1]-w[1]*y[0]+w[1]*y[2]-w[2]*y[1]+w[2]*y[0]-w[0]*y[2])/delta)
                parX.append((x[0]*w[1]-x[1]*w[0]+x[1]*w[2]-x[2]*w[1]+x[2]*w[0]-x[0]*w[2])/delta)
                parX.append((x[0]*y[1]*w[2]-x[1]*y[0]*w[2]+x[1]*y[2]*w[0]-x[2]*y[1]*w[0]+x[2]*y[0]*w[1]-x[0]*y[2]*w[1])/delta)
                parY=[]
                parY.append((z[0]*y[1]-z[1]*y[0]+z[1]*y[2]-z[2]*y[1]+z[2]*y[0]-z[0]*y[2])/delta)
                parY.append((x[0]*z[1]-x[1]*z[0]+x[1]*z[2]-x[2]*z[1]+x[2]*z[0]-x[0]*z[2])/delta)
                parY.append((x[0]*y[1]*z[2]-x[1]*y[0]*z[2]+x[1]*y[2]*z[0]-x[2]*y[1]*z[0]+x[2]*y[0]*z[1]-x[0]*y[2]*z[1])/delta)
                x1=parX[0]*nodo.GetX()/self.frame.Zoom+parX[1]*nodo.GetY()/self.frame.Zoom+parX[2]
                y1=parY[0]*nodo.GetX()/self.frame.Zoom+parY[1]*nodo.GetY()/self.frame.Zoom+parY[2]
                msg=wx.MessageDialog(self.frame, "Point value`: ("+str(x1)+","+str(y1)+")", style=wx.YES_NO)
                val=msg.ShowModal()
                msg.Destroy()
                if val==wx.ID_YES:
                    self.frame.parX=parX
                    self.frame.parY=parY
                    self.frame.EnableItems([100,101,110,120,121,130,131,140,141,150,151,160,161,170,171,180,181])
                    self.frame.ToggleItems([])
                else:
                    msg=wx.MessageDialog(self.frame, "Procedure canceled!", style=wx.OK)
                    msg.ShowModal()
                    msg.Destroy()
                    self.frame.ToggleItems([])
                for ss in self.diagram.GetShapeList()[:]:
                    if ss.tipo=="Assi":
                        self.diagram.RemoveShape(ss)
                self.canvas.Redraw(self.dc)
                self.frame.w=[]
                self.frame.z=[]

    def InsNodo(self, x, y):
        nodo=self.frame.AddNodo(x, y, wx.RED)
        self.SetStatusbar(nodo)
        self.canvas.Redraw(self.dc)
        return nodo

    def InsLinea(self, x, y):
        self.InsPoligonale(x, y)
        if len(self.frame.SplineNode)>1:
            self.frame.SplineNode=[]
            self.frame.SplineColor=self.frame.MakeColour()

    def InsPoligonale(self, x, y):
        if self.shape.tipo=="Background":
            nodo=self.InsNodo(x, y)
        else:
            nodo=self.shape
            self.SetStatusbar(nodo)
        if nodo.GetPen().GetColour()==wx.RED:
            self.frame.SplineNode.append(nodo)
            if len(self.frame.SplineNode)>1:
                if self.frame.SplineNode[-2]!=self.frame.SplineNode[-1]:
                    linea=self.frame.AddLinea(self.frame.SplineNode[-2], self.frame.SplineNode[-1], self.frame.SplineColor)
                    if len(self.frame.SplineNode)>2:
                        self.frame.SplineNode[-2].SetPen(wx.Pen((128,0,0), 1))
                        self.frame.SplineNode[-2].SetBrush(wx.Brush((128,0,0)))
                if len(nodo.GetLines())>1:
                    self.frame.SplineNode=[]
                    self.frame.SplineColor=self.frame.MakeColour()
            self.canvas.Redraw(self.dc)

    def DelShape(self):
        if self.shape.tipo=="Nodo":
            for ll in self.shape.GetLines()[:]:
                colour=ll.GetBrush().GetColour()
                newcolour=self.frame.MakeColour()
                stato=False
                for l2 in self.diagram.GetShapeList()[:]:
                    if l2.GetBrush().GetColour()==colour and stato==True:
                        l2.SetPen(wx.Pen(newcolour, 3))
                    if ll==l2: stato=True
                ll.GetFrom().SetPen(wx.Pen(wx.RED, 1))
                ll.GetFrom().SetBrush(wx.Brush(wx.RED))
                ll.GetTo().SetPen(wx.Pen(wx.RED, 1))
                ll.GetTo().SetBrush(wx.Brush(wx.RED))
                self.diagram.RemoveShape(ll)
            for dd in self.diagram.GetShapeList()[:]:
                if dd.tipo=="Dominio":
                    if dd.nodi.count(self.shape):
                        self.diagram.RemoveShape(dd)
        self.diagram.RemoveShape(self.shape)
        self.canvas.Redraw(self.dc)

    def InsDominio(self, verso):
        self.frame.PuntiDominio.append(self.shape)
        if len(self.frame.PuntiDominio)>1:
            val=False
            for l1 in self.frame.PuntiDominio[0].GetLines():
                for l2 in self.frame.PuntiDominio[1].GetLines():
                    if l1==l2:val=True
            if val:
                ciclo=True
                sumang=0
                while ciclo:
                    p0=self.frame.PuntiDominio[-2]
                    p1=self.frame.PuntiDominio[-1]
                    ln=p1.GetLines()
                    p2=[]
                    for lx in ln:
                        if lx.GetFrom()!=p1 and lx.GetFrom()!=p0: p2.append(lx.GetFrom())
                        elif lx.GetTo()!=p1 and lx.GetTo()!=p0: p2.append(lx.GetTo())
                    v1=((p1.GetX()-p0.GetX())/math.sqrt((p1.GetX()-p0.GetX())**2+(p1.GetY()-p0.GetY())**2),(p1.GetY()-p0.GetY())/math.sqrt((p1.GetX()-p0.GetX())**2+(p1.GetY()-p0.GetY())**2))
                    v2=[]
                    scp=[]
                    cxp=[]
                    angle=[]
                    for px in p2:
                        v2.append(((px.GetX()-p1.GetX())/math.sqrt((px.GetX()-p1.GetX())**2+(px.GetY()-p1.GetY())**2),(px.GetY()-p1.GetY())/math.sqrt((px.GetX()-p1.GetX())**2+(px.GetY()-p1.GetY())**2)))
                        scp.append(v1[0]*v2[-1][0]+v1[1]*v2[-1][1])
                        if verso==1:
                            cxp.append(v1[1]*v2[-1][0]-v1[0]*v2[-1][1])
                        else:
                            cxp.append(v1[0]*v2[-1][1]-v1[1]*v2[-1][0])
                        if cxp[-1]<0:sign=-1
                        elif cxp[-1]>0:sign=1
                        else: sign=0
                        angle.append(math.acos(scp[-1])*sign)
                    maxi=(-math.pi,0)
                    for alpha in angle:
                        if alpha>maxi[0]:maxi=(alpha,angle.index(alpha))
                    if p2[maxi[1]]==self.frame.PuntiDominio[0]:ciclo=False
                    self.frame.PuntiDominio.append(p2[maxi[1]])
                    sumang=sumang+maxi[0]
                vertici=[]
                for pp in self.frame.PuntiDominio:
                    vertici.append((pp.GetX(),pp.GetY()))
                vertici.pop()
                self.frame.PuntiDominio.pop()
                if sumang>0:
                    shpoly=self.frame.AddDominio(vertici, self.frame.PuntiDominio, verso, self.frame.MakeColour())
                    self.canvas.Redraw(self.dc)
                else:
                    msg=wx.MessageDialog(self.frame, "Domain not found!", style=wx.OK)
                    msg.ShowModal()
                    msg.Destroy()
            else:
                msg=wx.MessageDialog(self.frame, "Domain not found!", style=wx.OK)
                msg.ShowModal()
                msg.Destroy()
            self.frame.PuntiDominio=[]

    def SetVelocita(self):
        inp=GeoInputDialog(self.frame, "Set velocity", ["V:"], [self.shape.velocita])
        val=inp.ShowModal()
        res=inp.Destroy()
        if val==wx.ID_OK:
            self.shape.velocita=res[0]

    def SetGrid(self, x, y):
        pass
        # not yet implemented

    def SetStatusbar(self, nodo):
        x=(self.frame.parX[0]*nodo.GetX()+self.frame.parX[1]*nodo.GetY()+self.frame.parX[2])/self.frame.Zoom
        y=(self.frame.parY[0]*nodo.GetX()+self.frame.parY[1]*nodo.GetY()+self.frame.parY[2])/self.frame.Zoom
        if nodo.tipo=='Nodo':
            self.frame.SetStatusText('[Node]   X: '+str(x)+'   Y: '+str(y)+'   Velocity: '+str(nodo.velocita))
        elif nodo.tipo=='Dominio':
            self.frame.SetStatusText('[Domain]   Velocity: '+str(nodo.velocita))

#----------------------------------------------------------------------

class GeomodDesignFrame(wx.Frame):

    def __init__(self, parent=None, pos=(0,0), size=(800,600), counter="", path=None):
        wx.Frame.__init__(self, parent, -1, "GeomodVi"+counter+" - input parameters", pos=pos, size=size, style=wx.DEFAULT_FRAME_STYLE)
        ogl.OGLInitialize()
        self.parent=parent
        self.pos=pos
        self.size=size
        self.counter=counter
        if path==None:
            self.path=os.path.dirname(sys.argv[0])
            if self.path!="": self.path += "/"
        else:
            self.path=path
        self.resfolder="resources/"
        self.formula="vel/10*6/5.2"
        self.__MakeCanvas()
        self.__MakeToolbar()
        self.CreateStatusBar()
        self.SetMenu()
        self.Bind(wx.EVT_ACTIVATE, self.OnWindowActivate)
        
        self.mapshape=None
        self.mapcopy=wx.Image(self.path+self.resfolder+"grid.gif", wx.BITMAP_TYPE_GIF)
        image=self.mapcopy.Copy()
        self.mapshape=self.AddMap(image, "")
        self.EnableItems([100,110,120,121,180,181])
        
        self.Show(True)

    def SetMenu(self):
        self.menu=self.MakeMenu()

    def MakeMenu(self):
        menu_file = wx.Menu()
        FExit = menu_file.Append(-1, 'E&xit', 'Quit from DataVi')
        menu_help = wx.Menu()
        HIndex = menu_help.Append(-1, '&Help Index', 'Help of this applicantion...')
        HContacts = menu_help.Append(-1, '&Contacts', 'Contacts')
        mainmenu = wx.MenuBar()
        mainmenu.Append(menu_file, '&File')
        mainmenu.Append(menu_help, '&Help')
        self.Bind(wx.EVT_MENU, self.OnFileExit, FExit)
        self.Bind(wx.EVT_MENU, self.OnHelpIndex, HIndex)
        self.Bind(wx.EVT_MENU, self.OnHelpContacts, HContacts)
        return mainmenu

    def OnWindowActivate(self, event):
        self.SetMenuBar(self.menu)
        event.Skip()

    def __MakeCanvas(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.Canvas = ogl.ShapeCanvas(self)
        sizer.Add(self.Canvas, 1, wx.GROW)
        self.Canvas.SetBackgroundColour("LIGHT BLUE")
        self.Diagram = ogl.Diagram()
        self.Canvas.SetDiagram(self.Diagram)
        self.Diagram.SetCanvas(self.Canvas)
        self.SetSizer(sizer)
        self.DC=wx.PaintDC(self.Canvas)
        self.SetAutoLayout(True)

    def __MakeToolbar(self):
        
        id=['T100','T101','T110','T120','T121','T130','T131','T140','T141','T150','T151','T160','T161','T170','T171','T172','T180','T181','T182']
        for i in id:
            exec('self.'+i+'=wx.NewId()')
        self.Toolbar = self.CreateToolBar( wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT )
        self.Toolbar.SetToolBitmapSize((32,32))
        item=self.Toolbar.AddSimpleTool(self.T100, wx.Bitmap(self.path+self.resfolder+"gv01.jpg",wx.BITMAP_TYPE_JPEG), "Open geo-model")
        wx.EVT_TOOL(self, self.T100, self.OnOpen)
        item=self.Toolbar.AddSimpleTool(self.T101, wx.Bitmap(self.path+self.resfolder+"gv02.jpg",wx.BITMAP_TYPE_JPEG), "Save geo-model")
        wx.EVT_TOOL(self, self.T101, self.OnSave)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddSimpleTool(self.T110, wx.Bitmap(self.path+self.resfolder+"gv03.jpg",wx.BITMAP_TYPE_JPEG), "New geo-model")
        wx.EVT_TOOL(self, self.T110, self.OnClearAll)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddSimpleTool(self.T120, wx.Bitmap(self.path+self.resfolder+"gv04.jpg",wx.BITMAP_TYPE_JPEG), "Import map")
        wx.EVT_TOOL(self, self.T120, self.OnLoadMap)
        item=self.Toolbar.AddCheckTool(self.T121, wx.Bitmap(self.path+self.resfolder+"gv05.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Define frame coordinates")
        wx.EVT_TOOL(self, self.T121, self.OnSetAssi)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddCheckTool(self.T130, wx.Bitmap(self.path+self.resfolder+"gv06.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Insert node")
        wx.EVT_TOOL(self, self.T130, self.OnInsNodo)
        item=self.Toolbar.AddCheckTool(self.T131, wx.Bitmap(self.path+self.resfolder+"gv07.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Delete node")
        wx.EVT_TOOL(self, self.T131, self.OnDelNodo)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddCheckTool(self.T140, wx.Bitmap(self.path+self.resfolder+"gv10.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Insert line")
        wx.EVT_TOOL(self, self.T140, self.OnInsLinea)
        item=self.Toolbar.AddCheckTool(self.T141, wx.Bitmap(self.path+self.resfolder+"gv11.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Insert polyline")
        wx.EVT_TOOL(self, self.T141, self.OnInsPoligonale)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddCheckTool(self.T150, wx.Bitmap(self.path+self.resfolder+"gv12.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Define sub-domain (+)")
        wx.EVT_TOOL(self, self.T150, self.OnInsDominioOrario)
        item=self.Toolbar.AddCheckTool(self.T151, wx.Bitmap(self.path+self.resfolder+"gv13.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Define sub-domain (-)")
        wx.EVT_TOOL(self, self.T151, self.OnInsDominioAntiorario)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddSimpleTool(self.T160, wx.Bitmap(self.path+self.resfolder+"gv14.jpg",wx.BITMAP_TYPE_JPEG), "Define grid step criterion")
        wx.EVT_TOOL(self, self.T160, self.OnSetParametri)
        item=self.Toolbar.AddCheckTool(self.T161, wx.Bitmap(self.path+self.resfolder+"gv15.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Input parameter value")
        wx.EVT_TOOL(self, self.T161, self.OnSetVelocita)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddSimpleTool(self.T170, wx.Bitmap(self.path+self.resfolder+"gv16.jpg",wx.BITMAP_TYPE_JPEG), "Preview geo-model data file")
        wx.EVT_TOOL(self, self.T170, self.OnPreviewEsporta)
        item=self.Toolbar.AddSimpleTool(self.T171, wx.Bitmap(self.path+self.resfolder+"gv17.jpg",wx.BITMAP_TYPE_JPEG), "Export geo-model data file")
        wx.EVT_TOOL(self, self.T171, self.OnEsporta)
        #item=self.Toolbar.AddCheckTool(self.T172, wx.Bitmap(self.path+self.resfolder+"gv18.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Export geo-model grid file")
        item=self.Toolbar.AddCheckTool(self.T172, wx.Bitmap(self.path+self.resfolder+"gv18.jpg",wx.BITMAP_TYPE_JPEG), shortHelp="Export NOT yet implemented")
        wx.EVT_TOOL(self, self.T172, self.OnEsportaGrid)
        self.Toolbar.AddSeparator()
        item=self.Toolbar.AddSimpleTool(self.T180, wx.Bitmap(self.path+self.resfolder+"gv08.jpg",wx.BITMAP_TYPE_JPEG), "Zoom In")
        wx.EVT_TOOL(self, self.T180, self.OnZoomIn)
        item=self.Toolbar.AddSimpleTool(self.T181, wx.Bitmap(self.path+self.resfolder+"gv09.jpg",wx.BITMAP_TYPE_JPEG), "Zoom Out")
        wx.EVT_TOOL(self, self.T181, self.OnZoomOut)
        self.Toolbar.items=[100,101,110,120,121,130,131,140,141,150,151,160,161,170,171,172,180,181]
        self.Toolbar.Realize()
        self.EnableItems([100,120])

    def EnableItems(self, items):
        for i in self.Toolbar.items:
            if items.count(i): self.Toolbar.EnableTool(eval('self.T'+str(i)),True)
            else: self.Toolbar.EnableTool(eval('self.T'+str(i)),False)

    def ToggleItems(self, items):
        for i in self.Toolbar.items:
            if items.count(i): pass #self.Toolbar.ToggleTool(eval('self.T'+str(i)),True)
            else: self.Toolbar.ToggleTool(eval('self.T'+str(i)),False)

    def __AddShape(self, shape, x, y, pen, brush, text, drag, tipo):
        shape.tipo=tipo
        shape.SetDraggable(drag, True)
        shape.SetCanvas(self.Canvas)
        if x>=0:shape.SetX(x)
        if y>=0:shape.SetY(y)
        if pen: shape.SetPen(pen)
        if brush: shape.SetBrush(brush)
        if text: shape.AddText(text)
        self.Diagram.AddShape(shape)
        shape.Show(True)
        evthandler = GeoEvent(frame=self)
        evthandler.SetShape(shape)
        evthandler.SetPreviousHandler(shape.GetEventHandler())
        shape.SetEventHandler(evthandler)
        shape.x=x/self.Zoom
        shape.y=y/self.Zoom
        return shape

    def __AddLine(self, line, nodo1, nodo2, pen, brush, text, drag, tipo):
        line.tipo=tipo
        line.SetDraggable(drag, True)
        line.SetCanvas(self.Canvas)
        if pen: line.SetPen(pen)
        if brush: line.SetBrush(brush)
        if text: line.AddText(text)
        line.MakeLineControlPoints(2)
        nodo1.AddLine(line, nodo2)
        self.Diagram.AddShape(line)
        line.Show(True)
        evthandler = GeoEvent(frame=self)
        evthandler.SetShape(line)
        evthandler.SetPreviousHandler(line.GetEventHandler())
        line.SetEventHandler(evthandler)
        nodo1.Move(wx.PaintDC(self.Canvas), nodo1.GetX(), nodo1.GetY())
        return line

    def AddMap(self, image, filename):
        imgX=image.GetWidth()
        imgY=image.GetHeight()
        #if imgX>4000:
        #    self.Zoom=4000.0/imgX
        #    msg=wx.MessageDialog(self,\
        #    "Immagine troppo grande per esser visualizzata a dimensione reale"\
        #     +"\nDimensione massima orizzontale consentita: 4000 pixel"\
        #     +"\nZoom impostato al valore di:"+str(round(self.Zoom*100,1))+"%", style=wx.OK)
        #    msg.ShowModal()
        #    msg.Destroy()
        #    image.Rescale(int(imgX*self.Zoom),int(imgY*self.Zoom))
        #else:
        self.Zoom=1.0
        image=image.ConvertToBitmap()
        imgX=image.GetWidth()
        imgY=image.GetHeight()
        if self.mapshape!=None:
            shapeBMP=self.mapshape
            shapeBMP.SetX(imgX/2)
            shapeBMP.SetY(imgY/2)
            shapeBMP.x=imgX/2/self.Zoom
            shapeBMP.y=imgY/2/self.Zoom
        else:
            shapeBMP=ogl.BitmapShape()
        shapeBMP.SetBitmap(image)
        shapeBMP.SetFilename(filename)
        self.Canvas.SetScrollbars(1, 1, imgX, imgY)
        return self.__AddShape(shapeBMP, imgX/2, imgY/2, None, None, "",False,"Background")

    def AddAssi(self, x, y):
        return self.__AddShape(ogl.CircleShape(7), x, y, wx.Pen(wx.BLUE, 1), wx.Brush(wx.BLUE), "", True, "Assi")

    def AddNodo(self, x, y, colore, velocita='#'):
        ss=self.__AddShape(ogl.RectangleShape(7,7), x, y, wx.Pen(colore, 1), wx.Brush(colore), "", True, "Nodo")
        ss.velocita=velocita
        return ss

    def AddLinea(self, nodo1, nodo2, colore):
        return self.__AddLine(ogl.LineShape(), nodo1, nodo2, wx.Pen(colore, 2), wx.Brush(colore), "", False, "Linea")

    def AddDominio(self, vertici, nodi, verso, colore, velocita='#'):
        poly=ogl.PolygonShape()
        poly.Create(vertici)
        ss=self.__AddShape(poly, -1, -1, wx.Pen(colore, 1), wx.Brush(colore), "", False, "Dominio")
        ss.nodi=nodi
        ss.verso=verso
        ss.velocita=velocita
        return ss

    def ClearCanvas(self):
        self.Diagram.DeleteAllShapes()
        self.Diagram.Clear(wx.PaintDC(self.Canvas))
        self.parX=[1, 0, 0]
        self.parY=[0, 1, 0]
        self.Canvas.SetScrollbars(1, 1, 1, 1)

    def MakeColour(self):
        stato=True
        while stato:
            stato=False
            colour=wx.Colour(int(256*random.random()), int(256*random.random()), int(256*random.random()))
            for shape in self.Diagram.GetShapeList():
                if shape.GetBrush().GetColour()==colour: stato=True
        return colour

    def EsportaBl2d(self):
        ll=self.Diagram.GetShapeList()
        intestazione="-->> GeomodVi:  geo-model data file - (BL2D format) <<"
        nodi="\n--nodes"
        linee="\n--lines"
        domini='\n--domains'
        metriche="\n--discrete metric maps\nisotropic"
        puntiimposti="\n--constrained points"
        lcolore=(-1,-1,-1)
        for ss in ll:
            if ss.tipo=="Nodo":
                nodi=nodi+"\nP"+str(ll.index(ss))+"  "+str(ss.GetX())+"  "+str(ss.GetY())
                try:
                    velocita=float(ss.velocita)
                except:
                    velocita=ss.velocita
                if velocita=='#':
                    dd=[]
                    vd=True
                    for xx in ll:
                        if xx.tipo=="Dominio":
                            if xx.nodi.count(ss):
                                try:
                                    dd.append(float(xx.velocita))
                                except:
                                    dd.append(xx.velocita)
                                    vd=False
                    for vxx in dd:
                        if vd:
                            if velocita=='#': velocita=vxx
                            if velocita>vxx: velocita=vxx
                        else:
                            velocita+=' '+str(vxx)
                try:
                    metrica=str(eval(string.replace(self.formula,"vel",str(velocita))))
                except:
                    metrica=velocita
                metriche=metriche+"\nP"+str(ll.index(ss))+"  "+metrica
                trovato=False
                for k in ss.GetLines():
                    if string.count(ll,k): trovato=True
                if trovato==False: puntiimposti=puntiimposti+"\nP"+str(ll.index(ss))
            elif ss.tipo=="Linea":
                if lcolore!=ss.GetPen().GetColour():
                    if lcolore!=(-1,-1,-1): linee=linee+"  NULL ;"
                    lcolore=ss.GetPen().GetColour()
                    linee=linee+"\nL"+str(ll.index(ss))+"  NULL  P"+str(ll.index(ss.GetFrom()))
                linee=linee+"  P"+str(ll.index(ss.GetTo()))
            elif ss.tipo=="Dominio":
                vert=ss.nodi
                trovato=False
                for i in range(len(vert)-2):
                    for l1 in vert[i].GetLines():
                        for l2 in vert[i+1].GetLines():
                            if l1==l2 and string.count(linee,"\nL"+str(ll.index(l1))+" ")>0 and trovato==False:
                                if l1.GetTo()==vert[i]: verso=ss.verso
                                else: verso=-ss.verso
                                domini=domini+"\nL"+str(ll.index(l1))+"  "+str(verso)+"  "+str(ll.index(ss))
                                trovato=True
        if linee!="": linee=linee+"  NULL ;"
        intestazione=intestazione+"\n"+str(string.count(nodi,"\nP"))+"  "+str(string.count(linee,"\nL"))+"  "+str(string.count(domini,"\nL"))
        nodi=nodi+"\n;"
        puntiimposti=puntiimposti+"\n;"
        linee=linee+"\n;"
        reffisnodi="\n--nodes references\n;"
        reffislinee="\n--lines references\n;"
        domini=domini+"\n;"
        metriche=metriche+"\n;"
        text=intestazione+nodi+puntiimposti+linee+reffisnodi+reffislinee+domini+metriche
        return text

    def EsportaGrid(self, xsteps, ysteps):
        ll=self.Diagram.GetShapeList()
        slim=[]
        sdom=[]
        smetr=[]
        for ss in ll:
            if ss.tipo=="Assi": slim.append(ss)
            elif ss.tipo=="Dominio":
                sdom.append(ss)
                try: smetr.append(eval(string.replace(self.formula,"vel",str(ss.velocita))))
                except: smetr.append(ss.velocita)
        xmin=min(slim[-1].GetX(),slim[-2].GetX())
        xmax=max(slim[-1].GetX(),slim[-2].GetX())
        ymin=min(slim[-1].GetY(),slim[-2].GetY())
        ymax=max(slim[-1].GetY(),slim[-2].GetY())
        matrix=[]
        for j in range(ysteps):
            y=j/float(ysteps-1)*(ymax-ymin)+ymin
            row=[]
            for i in range(xsteps):
                x=i/float(xsteps-1)*(xmax-xmin)+xmin
                for ss in sdom:
                    vv=[]
                    angle=0
                    for nn in ss.nodi:
                        denom=math.sqrt((nn.GetX()-x)**2+(nn.GetY()-y)**2)
                        vv.append(((nn.GetX()-x)/denom,(nn.GetY()-y)/denom))
                    vv.append(vv[0])
                    for k in range(len(vv)-1):
                        scp=vv[k][0]*vv[k+1][0]+vv[k][1]*vv[k+1][1]
                        cxp=vv[k][0]*vv[k+1][1]-vv[k][1]*vv[k+1][0]
                        angle+=math.acos(scp)*num.sign(cxp)
                    if angle>1 or angle<-1:
                        val=smetr[sdom.index(ss)]
                        break
                    else: val=0
                row.append(val)
            matrix.append(row)
        return matrix

    def OnOpen(self, event):
        dlg = wx.FileDialog(self, "Open GeoModel file...", self.path,
                           style=wx.OPEN, wildcard = "GeoModel files (*.geom)|*.geom|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            self.Canvas.PrepareDC(wx.PaintDC(self.Canvas))
            self.ClearCanvas()
            filename = dlg.GetPath()
            text=[]
            if string.split(filename,".")[-1]=="geom":
                a=""
                for i in string.split(filename,".")[:-1]:
                    a+=i+"."
                filename=a[:-1]
            f=file(filename+".geom",'r')
            while 1:
                row=f.readline()
                if len(row)==0: break
                text.append(eval(row))
            for tt in text:
                if tt['Tipo']=='Background':
                    self.mapcopy=wx.Image(filename+".jpeg", wx.BITMAP_TYPE_JPEG)
                    image=self.mapcopy.Copy()
                    self.mapshape=self.AddMap(image, filename+".jpeg")
                    self.Canvas.parX=tt['parX']
                    self.Canvas.parY=tt['parY']
                    self.formula=tt['Formula']
                elif tt['Tipo']=='Nodo':
                    self.AddNodo(tt['X'], tt['Y'], tt['Colore'], tt['Velocita'])
                elif tt['Tipo']=='Linea':
                    ll=self.Diagram.GetShapeList()
                    self.AddLinea(ll[tt['From']], ll[tt['To']], tt['Colore'])
                elif tt['Tipo']=='Dominio':
                    ll=self.Diagram.GetShapeList()
                    PuntiDominio=[]
                    vertici=[]
                    for nn in tt['Nodi']:
                        PuntiDominio.append(ll[nn])
                        vertici.append((ll[nn].GetX(),ll[nn].GetY()))
                    self.AddDominio(vertici, PuntiDominio, tt['Verso'], tt['Colore'], tt['Velocita'])
            self.Canvas.Redraw(wx.PaintDC(self.Canvas))
            self.EnableItems([100,101,110,120,121,130,131,140,141,150,151,160,161,170,171,180,181])
            self.ToggleItems([])
        dlg.Destroy()

    def OnSave(self, event):
        dlg = wx.FileDialog(self, "Save GeoModel file...", self.path,
                           style=wx.SAVE, wildcard = "GeoModel files (*.geom)|*.geom|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            filename = dlg.GetPath()
            text=[]
            ll=self.Diagram.GetShapeList()
            for ss in ll:
                if ss.tipo=="Background":
                    text.append({'Tipo':'Background','parX':self.parX[:],'parY':self.parY[:],'Filename':ss.GetFilename(),'Formula':self.formula})
                elif ss.tipo=="Nodo":
                    text.append({'Tipo':'Nodo','X':ss.x,'Y':ss.y,'Colore':ss.GetPen().GetColour(),'Velocita':ss.velocita})
                elif ss.tipo=="Linea":
                    text.append({'Tipo':'Linea','From':ll.index(ss.GetFrom()),'To':ll.index(ss.GetTo()),'Colore':ss.GetPen().GetColour()})
                elif ss.tipo=="Dominio":
                    nodi=[]
                    for nn in ss.nodi:
                        nodi.append(ll.index(nn))
                    text.append({'Tipo':'Dominio','Nodi':nodi,'Verso':ss.verso,'Colore':ss.GetPen().GetColour(),'Velocita':ss.velocita})
            if string.split(filename,".")[-1]=="geom":
                a=""
                for i in string.split(filename,".")[:-1]:
                    a+=i+"."
                filename=a[:-1]
            f=file(filename+".geom",'w')
            for tt in text:
                f.write(str(tt)+'\n')
            f.close()
            self.mapshape.GetBitmap().ConvertToImage().SaveFile(filename+".jpeg",wx.BITMAP_TYPE_JPEG)
        dlg.Destroy()

    def OnClearAll(self, event):
        #self.ClearCanvas()
        #self.ToggleItems([])
        #self.EnableItems([100,120])
        self.parent.OnGeomdVi(wx.Event)

    def OnLoadMap(self, event):
        dlg = wx.FileDialog(self, "Open image file...", self.path, style=wx.OPEN,
            wildcard = "Image Gif files (*.gif)|*.gif|Image JPEG files (*.jpg *.jpeg)|*.jpg;*.jpeg|Image XPM files (*.xpm)|*.xpm|Image Bitmap files (*.bmp)|*.bmp|Image PNG files (*.png)|*.png|Image TIFF files (*.tif *.tiff)|*.tif;*.tiff|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            filename = dlg.GetPath()
            if string.upper(filename[-3:])=="GIF":
                self.mapcopy=wx.Image(filename, wx.BITMAP_TYPE_GIF)
            elif string.upper(filename[-3:])=="JPG" or string.upper(filename[-4:])=="JPEG":
                self.mapcopy=wx.Image(filename, wx.BITMAP_TYPE_JPEG)
            elif string.upper(filename[-3:])=="XPM":
                self.mapcopy=wx.Image(filename, wx.BITMAP_TYPE_XPM)
            elif string.upper(filename[-3:])=="BMP":
                self.mapcopy=wx.Image(filename, wx.BITMAP_TYPE_BMP)
            elif string.upper(filename[-3:])=="PNG":
                self.mapcopy=wx.Image(filename, wx.BITMAP_TYPE_PNG)
            elif string.upper(filename[-3:])=="TIF" or string.upper(filename[-4:])=="TIFF":
                self.mapcopy=wx.Image(filename, wx.BITMAP_TYPE_TIFF)
            image=self.mapcopy.Copy()
            self.mapshape=self.AddMap(image, filename)
            self.EnableItems([100,110,120,121,180,181])
        dlg.Destroy()

    def OnSetAssi(self, event):
        self.ToggleItems([121])
        self.w=[]
        self.z=[]
        for ss in self.Diagram.GetShapeList():
            if ss.tipo=="Assi":
                self.Diagram.RemoveShape(ss)
                #ss.Destroy()
        self.Canvas.Redraw(wx.ClientDC(self.Canvas))

    def OnInsNodo(self, event):
        self.ToggleItems([130])

    def OnDelNodo(self, event):
        self.ToggleItems([131])

    def OnInsLinea(self, event):
        self.ToggleItems([140])
        self.SplineNode=[]
        if self.Toolbar.GetToolState(self.T140): self.SplineColor=self.MakeColour()

    def OnInsPoligonale(self, event):
        self.ToggleItems([141])
        self.SplineNode=[]
        if self.Toolbar.GetToolState(self.T141): self.SplineColor=self.MakeColour()

    def OnInsDominioOrario(self, event):
        self.ToggleItems([150])
        self.PuntiDominio=[]

    def OnInsDominioAntiorario(self, event):
        self.ToggleItems([151])
        self.PuntiDominio=[]

    def OnSetParametri(self, event):
        par=GeoParDialog(self,self.formula)
        val=par.ShowModal()
        res=par.Destroy()
        if val==wx.ID_OK: self.formula=res

    def OnSetVelocita(self, event):
        self.ToggleItems([161])
        self.PuntiDominio=[]

    def OnPreviewEsporta(self, event):
        text=self.EsportaBl2d()
        dlg=GeoText(self, text)
        dlg.ShowModal()
        dlg.Destroy()

    def OnEsporta(self, event):
        dlg = wx.FileDialog(self, "Save BL2D file...", self.path,
                           style=wx.SAVE, wildcard = "Data Bl2d files (*.data)|*.data|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            filename = dlg.GetPath()
            f=file(filename,'w')
            f.write(self.EsportaBl2d())
            f.close()
        dlg.Destroy()

    def OnEsportaGrid(self, event):
        self.ToggleItems([172])
        self.gridcount=0
        for ss in self.Diagram.GetShapeList():
            if ss.tipo=="Assi":
                self.Diagram.RemoveShape(ss)
                #ss.Destroy()
        self.Canvas.Redraw(wx.PaintDC(self.Canvas))

    def OnZoomIn(self, event):
        if self.mapcopy.GetWidth()*self.Zoom*1.25<=4000:
            self.Zoom=self.Zoom*1.25
            self.SetZoom()

    def OnZoomOut(self, event):
        if self.mapcopy.GetWidth()*self.Zoom*0.8<=4000:
            self.Zoom=self.Zoom*0.8
            self.SetZoom()

    def SetZoom(self):
        image=self.mapcopy.Copy()
        image.Rescale(int(image.GetWidth()*self.Zoom),int(image.GetHeight()*self.Zoom))
        image=image.ConvertToBitmap()
        self.mapshape.SetBitmap(image)
        imgX=image.GetWidth()
        imgY=image.GetHeight()
        self.mapshape.SetX(imgX/2)
        self.mapshape.SetY(imgY/2)
        self.Canvas.SetScrollbars(1, 1, imgX, imgY)
        for ss in self.Diagram.GetShapeList():
            if ss.tipo=="Assi" or ss.tipo=="Nodo":
                ss.Move(wx.PaintDC(self.Canvas),ss.x*self.Zoom,ss.y*self.Zoom)
            elif ss.tipo=="Dominio":
                lnn=[]
                for nn in ss.nodi:
                    lnn.append((nn.GetX(),nn.GetY()))
                    ss.Create(lnn)
        self.Canvas.Redraw(wx.PaintDC(self.Canvas))

    def OnFileExit(self, event):
        self.Close()

    def OnHelpIndex(self, event):
        svcom.HelpView(self)

    def OnHelpContacts(self, event):
        svcom.Contacts(self)

def main(*args, **kwds):
    app=wx.PySimpleApp()
    GeomodDesignFrame(*args, **kwds)
    app.MainLoop()

if __name__=='__main__':
    main()
