#! /usr/bin/env python
#
# SismoVi, version 2.00, December 2005.
#      
# Copyright (C) 2004-2005, Francesco Apostolico, Geza Seriani
# This program is free software you can redistribute it and/or 
#     modify it under the terms of the GNU General Public License 
#     as published by the Free Software Foundation either version 2
#     of the License, or any later version, with the following conditions
#     attached in addition to any and all conditions of the GNU
#     General Public License:
#     When reporting or displaying any results or animations created
#     using this code or modification of this code, make the appropriate
#     citation referencing SismoVi by name and including the version
#     number.
#
# This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY without even the implied warranty 
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#     along with this program if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
#     02111-1307  USA
#
# Authors:
#
# Francesco Apostolico, Geza Seriani
# Istituto Nazionale di Oceanografia e di Geofisica Sperimentale - OGS
# Borgo Grotta Gigante, 42/C, Sgonico
# 34010, Trieste, Italy
#
# fapostolico@inogs.it, gseriani@inogs.it
# 
# #######################################/
# SismoVi is a tool for the pre and post processing of data that are used in
#      or generated by seismic modeling.
#      It is platform independent and it is based on 
#      Python 2.3 (http://www.python.org/),
#      numarray 1.3 (http://www.stsci.edu/resources/software_hardware/numarray) and
#      WxPython 2.6 (http://wxpython.org/)
#      that must be installed in your platform before running it. 
#      An interactive help is available when running SismoVi.
#---------------------------------------------------------------------------
###### meshvi.py

import wx
import wx.grid
import wx.lib.scrolledpanel as scrolled
import wx.lib.colourselect as csel
import numarray.random_array as rand
import wx.tools.helpviewer as helpv
import wx.lib.plot as plot
import commonvi as svcom
import numarray as num
import os
import string
import sys

class MeshParamsDialog(wx.Frame):
    def __init__(self, parent=None, pos=(0,0), size=(-1,-1), counter="", path=None):
        wx.Frame.__init__(self, parent, -1, "MeshVi"+counter+" - input parameters", pos=pos,
            style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
        self.parent=parent
        self.pos=pos
        self.size=size
        self.counter=counter
        if path==None:
            self.path=os.path.dirname(sys.argv[0])
            if self.path!="": self.path += "/"
        else:
            self.path=path

        self.__Layout()

        self.Show(True)

        self.lmap=svcom.LinearMapping()

        self.read_bl2d=svcom.Bl2dRead()
        self.write_bl2d=svcom.Bl2dWrite()

    def __Layout(self):

# make tabs
        self.tabs=wx.Notebook(self, -1)

# make scrolled-panels
        self.panel_data=scrolled.ScrolledPanel(self.tabs, -1)
        self.panel_visualization=scrolled.ScrolledPanel(self.tabs, -1)

# make buttons
        self.button_draw = wx.Button(self, -1, "Visualize")
        self.button_convert = wx.Button(self, -1, "Convert")
        self.button_hidden = wx.Button(self, -1, "Hidden")
        self.button_loadnodes = wx.Button(self.panel_data, -1, "Show")
        self.button_loadelements = wx.Button(self.panel_data, -1, "Show")

# make sliders

        #self.slider_nodes = wx.Slider(self.panel_data, -1, style=wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS)
        #self.slider_elements = wx.Slider(self.panel_data, -1, style=wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS)

# make grids for data_panel
        self.grid_nodes = wx.grid.Grid(self.panel_data, -1)
        self.grid_nodes.CreateGrid(9,2)
        self.grid_nodes.SetColLabelValue(0,'X')
        self.grid_nodes.SetColLabelValue(1,'Y')
        self.grid_elements = wx.grid.Grid(self.panel_data, -1)
        self.grid_elements.CreateGrid(9,5)
        self.grid_elements.SetColLabelValue(0,'V1')
        self.grid_elements.SetColLabelValue(1,'V2')
        self.grid_elements.SetColLabelValue(2,'V3')
        self.grid_elements.SetColLabelValue(3,'V4')
        self.grid_elements.SetColLabelValue(4,'Domain')

# make label and text for data_panel
        self.label_nodes = wx.StaticText(self.panel_data, -1, "Nodes:")
        font=self.label_nodes.GetFont()
        font.SetWeight(wx.BOLD)
        self.label_nodes.SetFont(font)
        self.label_elements = wx.StaticText(self.panel_data, -1, "Elements/Domains:")
        font=self.label_elements.GetFont()
        font.SetWeight(wx.BOLD)
        self.label_elements.SetFont(font)
        self.label_loadnodes = wx.StaticText(self.panel_data, -1, "First node:")
        self.label_loadelements = wx.StaticText(self.panel_data, -1, "First element:")

# make label and text for data_panel
        self.label_domains = wx.StaticText(self.panel_visualization, -1, "View Domains:")

# make choice
        self.choice_domains = wx.Choice(self.panel_visualization)
        #self.choice_numnodes = wx.Choice(self.panel_data, choices=["10", "100", "1000"])
        #self.choice_numnodes.SetSelection(1)
        #self.choice_numelements = wx.Choice(self.panel_data, choices=["10", "100", "1000"])
        #self.choice_numelements.SetSelection(1)

# make radio
        self.radio_format = wx.RadioBox(self.panel_visualization, -1, "View Format:",
             choices=["Elements", "Nodes", "Nodes+Elements"],
             majorDimension=0, style=wx.RA_SPECIFY_COLS)

# make spin
        self.spin_loadnodes=wx.SpinCtrl(self.panel_data)
        self.spin_loadelements=wx.SpinCtrl(self.panel_data)

###### layout of objects in frame

# add sizer for nodes params
        self.sizer_data_nodes=wx.FlexGridSizer(0,4,0,0)
        #self.sizer_data_nodes.Add(self.slider_nodes,0,wx.ALIGN_LEFT)
        #self.sizer_data_nodes.Add(self.choice_numnodes,0,wx.ALIGN_LEFT)
        self.sizer_data_nodes.Add(self.label_loadnodes,0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        self.sizer_data_nodes.Add(self.spin_loadnodes,0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        self.sizer_data_nodes.Add(wx.StaticText(self.panel_data,-1,"x1000"),0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        self.sizer_data_nodes.Add(self.button_loadnodes,0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)

# add sizer for elements params
        self.sizer_data_elements=wx.FlexGridSizer(0,4,0,0)
        #self.sizer_data_elements.Add(self.slider_elements,0,wx.ALIGN_LEFT)
        #self.sizer_data_elements.Add(self.choice_numelements,0,wx.ALIGN_LEFT)
        self.sizer_data_elements.Add(self.label_loadelements,0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        self.sizer_data_elements.Add(self.spin_loadelements,0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        self.sizer_data_elements.Add(wx.StaticText(self.panel_data,-1,"x1000"),0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        self.sizer_data_elements.Add(self.button_loadelements,0,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)

# add main sizer for data_panel
        self.sizer_data_main=wx.FlexGridSizer(0,2,10,10)
        self.sizer_data_main.Add(self.label_nodes,0,wx.ALIGN_LEFT)
        self.sizer_data_main.Add(self.label_elements,0,wx.ALIGN_LEFT)
        self.sizer_data_main.Add(self.grid_nodes,0,wx.ALIGN_LEFT)
        self.sizer_data_main.Add(self.grid_elements,0,wx.ALIGN_LEFT)
        self.sizer_data_main.Add(self.sizer_data_nodes,0,wx.ALIGN_LEFT)
        self.sizer_data_main.Add(self.sizer_data_elements,0,wx.ALIGN_LEFT)
        self.panel_data.SetSizer(self.sizer_data_main)

# add choices
        self.sizer_choices=wx.FlexGridSizer(0,2,10,10)
        self.sizer_choices.Add(self.label_domains)
        self.sizer_choices.Add(self.choice_domains)

# add main view panel
        self.sizer_mainview=wx.FlexGridSizer(0,1,10,10)
        self.sizer_mainview.Add(self.sizer_choices)
        self.sizer_mainview.Add(self.radio_format)
        self.panel_visualization.SetSizer(self.sizer_mainview)

# add pages at tabs
        self.tabs.AddPage(self.panel_data, 'Data', True)
        self.tabs.AddPage(self.panel_visualization, 'Visualization', False)

# add button sizer
        self.sizer_buttons=wx.GridSizer(0,3,10,10)
        self.sizer_buttons.Add(self.button_draw)
        self.sizer_buttons.Add(self.button_convert)
        self.sizer_buttons.Add(self.button_hidden)

# add main sizer        
        self.sizer_main=wx.FlexGridSizer(0,1,0,0)
        self.sizer_main.Add(self.tabs, 0, wx.ALIGN_CENTER)
        self.sizer_main.Add((15,15))
        self.sizer_main.Add(self.sizer_buttons, 0, wx.ALIGN_CENTER)
        self.sizer_main.Add((15,15))
        self.SetSizer(self.sizer_main)
        self.SetAutoLayout(True)
        
        self.panel_data.Fit()
        self.sizer_main.Fit(self)

###### make manu
        menu_file = wx.Menu()
        FOpen = menu_file.Append(-1, 'Open Data...', 'Open file for plotting')
        FSave = menu_file.Append(-1, 'Save Data...', 'Save file for plotting')
        FExit = menu_file.Append(-1, 'E&xit', 'Quit from DataVi')
        menu_help = wx.Menu()
        HIndex = menu_help.Append(-1, '&Help Index', 'Help of this applicantion...')
        HContacts = menu_help.Append(-1, '&Contacts', 'Contacts')
        self.menu = wx.MenuBar()
        self.menu.Append(menu_file, '&File')
        self.menu.Append(menu_help, '&Help')

###### make events of objects in frame
        self.Bind(wx.EVT_ACTIVATE, self.OnWindowActivate)
        self.Bind(wx.EVT_BUTTON, self.OnWindowSubmit, self.button_draw)
        self.Bind(wx.EVT_BUTTON, self.OnConvert, self.button_convert)
        self.Bind(wx.EVT_BUTTON, self.OnChangeNodes, self.button_loadnodes)
        self.Bind(wx.EVT_BUTTON, self.OnChangeElements, self.button_loadelements)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.OnNodeCellChange, self.grid_nodes)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.OnElementCellChange, self.grid_elements)
        self.Bind(wx.EVT_MENU, self.OnFileOpen, FOpen)
        self.Bind(wx.EVT_MENU, self.OnFileSave, FSave)
        self.Bind(wx.EVT_MENU, self.OnFileExit, FExit)
        self.Bind(wx.EVT_MENU, self.OnHelpIndex, HIndex)
        self.Bind(wx.EVT_MENU, self.OnHelpContacts, HContacts)

    def SetParamsDialog(self, frame):
        self.params=frame

    def OnFileExit(self, event):
        self.Close()

    def OnHelpIndex(self, event):
        svcom.HelpView(self)
        
    def OnHelpContacts(self, event):
        svcom.Contacts(self)

    def OnWindowActivate(self, event):
        self.SetMenuBar(self.menu)
        event.Skip()

    def OnNodeCellChange(self, event):
        self.nodes[event.GetRow()+int(self.grid_nodes.GetRowLabelValue(0))-1,event.GetCol()]=\
                float(self.grid_nodes.GetCellValue(event.GetRow(),event.GetCol()))

    def OnWindowSubmit(self, event):
        self.wxPyPlotRender()

    def OnConvert(self, event):
        dd=['two','three','four','five','six','seven','eight','nine','ten','eleven','twelve','thirteen','fourteen']
        dlg = wx.SingleChoiceDialog(self, 'Choice:', 'Number of steps',dd,wx.CHOICEDLG_STYLE)
        if dlg.ShowModal() == wx.ID_OK:
            nn=dd.index(dlg.GetStringSelection())
            self.lmap.SetSteps(nsteps=nn)
            self.SetValues(self.lmap(self.nodes, self.elements))
        dlg.Destroy()

    def OnElementCellChange(self, event):
        self.elements[event.GetRow()+int(self.grid_elements.GetRowLabelValue(0))-1,event.GetCol()]=\
                float(self.grid_elements.GetCellValue(event.GetRow(),event.GetCol()))

    def OnChangeNodes(self, event):
        self.FillGridNodes()
    
    def OnChangeElements(self, event):
        self.FillGridElements()
    
    def SetValues(self,a):
        self.element_type=a[0]
        self.nodes=a[1]
        self.elements=a[2]
        self.choice_domains.Clear()
        self.choice_domains.Append('All')
        dom=num.arange(1,self.elements[:,-1].max()+1)
        for k in dom: self.choice_domains.Append(str(k))
        self.choice_domains.SetSelection(0)
        self.spin_loadnodes.SetRange(0,len(self.nodes)/1000)
        self.spin_loadnodes.SetValue(0)
        self.spin_loadelements.SetRange(0,len(self.elements)/1000)
        self.spin_loadelements.SetValue(0)
        self.grid_nodes.DeleteRows(0,self.grid_nodes.GetNumberRows())
        if len(self.nodes)>1000: numnodes=1000
        else: numnodes=len(self.nodes)
        self.grid_nodes.AppendRows(numnodes)
        self.FillGridNodes()
        self.grid_elements.DeleteRows(0,self.grid_elements.GetNumberRows())
        self.grid_elements.DeleteCols(0,self.grid_elements.GetNumberCols())
        if len(self.elements)>1000: numelements=1000
        else: numelements=len(self.elements)
        self.grid_elements.AppendRows(numelements)
        self.grid_elements.AppendCols(len(self.elements[0]))
        for i in range(self.grid_elements.GetNumberCols()):
            if i<4:
                self.grid_elements.SetColLabelValue(i,'V'+str(i+1))
            elif i<self.grid_elements.GetNumberCols()-1:
                self.grid_elements.SetColLabelValue(i,str(i-3))
            else:
                self.grid_elements.SetColLabelValue(i,'Domain')
        self.FillGridElements()
        
    def SearchElement(self, point):
        xx=self.frame.client.GetXCurrentRange()
        yy=self.frame.client.GetYCurrentRange()
        for k in range(len(self.nodes)):
            if num.abs(point[0]-self.nodes[k,0])<(xx[1]-xx[0])*0.01 and num.abs(point[1]-self.nodes[k,1])<(xx[1]-xx[0])*0.01:
                self.Show(False)
                self.Show(True)
                if k<=self.slider_nodes.GetMax():
                    self.slider_nodes.SetValue(k)
                else:
                    self.slider_nodes.SetValue(self.slider_nodes.GetMax())
                if self.slider_nodes.GetMax()>0: self.FillGridNodes(self.slider_nodes.GetValue())
                self.grid_nodes.SelectRow(k-int(self.grid_nodes.GetRowLabelValue(0))+1)
                self.grid_nodes.MakeCellVisible(k-int(self.grid_nodes.GetRowLabelValue(0))+1,0)
                break
        
    def FillGridNodes(self):
        numnodes=len(self.nodes)
        for j in range(self.grid_nodes.GetNumberRows()):
            for i in range(len(self.nodes[0])):
                if j+self.spin_loadnodes.GetValue()*1000<numnodes:
                    val=str(self.nodes[j+self.spin_loadnodes.GetValue()*1000][i])
                else:
                    val=""
                self.grid_nodes.SetCellValue(j,i,val)
            self.grid_nodes.SetRowLabelValue(j,str(j+self.spin_loadnodes.GetValue()*1000))

    def FillGridElements(self):
        numelements=len(self.elements)
        for j in range(self.grid_elements.GetNumberRows()):
            for i in range(len(self.elements[0])):
                if j+self.spin_loadelements.GetValue()*1000<numelements:
                    val=str(self.elements[j+self.spin_loadelements.GetValue()*1000][i])
                else:
                    val=""
                self.grid_elements.SetCellValue(j,i,val)
            self.grid_elements.SetRowLabelValue(j,str(j+self.spin_loadelements.GetValue()*1000))

    def wxPyPlotRender(self):
        element_type=self.element_type
        nodes=self.nodes
        elements=self.elements
        if self.choice_domains.GetString(self.choice_domains.GetSelection())=='All':
            domains=0
        else:
            domains=int(self.choice_domains.GetString(self.choice_domains.GetSelection()))
        format=self.radio_format.GetSelection()
        sel_elements=self.grid_elements.GetSelectedRows()
        col=num.reshape(rand.random(elements[:,-1].max()*3)*256,(elements[:,-1].max(),3))
        el=[]
        if domains: domains_cond=num.equal(elements[:,-1],domains)
        else: domains_cond=num.ones(len(elements[:,-1]))
        if sel_elements:
            arr=range(len(elements[:,-1]))
            elements_cond=num.zeros(len(elements[:,-1]))
            for k in sel_elements:
                elements_cond=num.equal(arr,k) | elements_cond
        else: elements_cond=num.ones(len(elements[:,-1]))
        viewel=num.compress(domains_cond & elements_cond,elements)
        if element_type[0:4]=='q1.1' and (format==0 or format==2):
            for k in viewel:
                el.append(svcom.PolyPolygon(nodes[k[0:4]-1], legend= '',\
                        colour=wx.Colour(255,255,255), fillcolour=wx.Colour(col[k[-1]-1,0],col[k[-1]-1,1],col[k[-1]-1,2])))
        elif element_type[0:4]=='q2.1' and (format==0 or format==2):
            for k in viewel:
                el.append(svcom.PolyPolygon(nodes[num.transpose(num.reshape(k[0:-1],(2,4))).flat-1], legend= '',\
                        colour=wx.Colour(255,255,255), fillcolour=wx.Colour(col[k[-1]-1,0],col[k[-1]-1,1],col[k[-1]-1,2])))
        if format==1 or format==2:
            for k in viewel:
                el.append(plot.PolyMarker(nodes[k[:-1]-1], legend= '', colour='black', fillcolour='white'))
        canvas=svcom.PlotFrame(self, pos=(self.pos[0]+33,self.pos[1]+33), size=self.size, counter=self.counter)
        canvas.client.Draw(plot.PlotGraphics(el,'', '', ''),\
                (self.nodes[:,0].min(),self.nodes[:,0].max()),(self.nodes[:,1].min(),self.nodes[:,1].max()))
        canvas.Show(True)

    def OnFileOpen(self, event):
        dlg = wx.FileDialog(self, "Open File...", self.path, style=wx.OPEN, wildcard = "*")
        if dlg.ShowModal() == wx.ID_OK:
            fname = dlg.GetPath()
            self.SetValues(self.read_bl2d(fname))
        dlg.Destroy()

    def OnFileSave(self, event):
        dlg = wx.FileDialog(self, "Save Simulator file...", self.path, style=wx.SAVE, wildcard = "Simulator files (*.mesh)|*.pick|All files (*.*)|*.*")
        if dlg.ShowModal() == wx.ID_OK:
            self.write_bl2d(dlg.GetPath(), self.params.element_type, self.params.nodes, self.params.elements)
        dlg.Destroy()

def main(*args, **kwds):
    app=wx.PySimpleApp()
    MeshParamsDialog(*args, **kwds)
    app.MainLoop()

if __name__=='__main__':
    main()
